/* 
-------------------------------------------------------------------------
./logicaland 0.1 - a participative global simulation
-------------------------------------------------------------------------
Copyright (c) 2002 ./logicaland 
Michael Aschauer, Josef Deinhofer (io@logicland.net)
-------------------------------------------------------------------------
For information on usage and redistribution, and for a DISCLAIMER OF ALL
WARRANTIES, see the file, "LICENSE.txt," in this distribution.  
-------------------------------------------------------------------------
*/



import logica.gui.*;
import logica.sim.*;
import logica.utility.*;

import java.awt.*;
import java.net.*;
import java.applet.*;


public class LogicalAnd 
extends Applet 
implements Runnable
{
	String[] classArray = {

		"logica.gui.LogicaControlCenter",
		"logica.gui.Style",
		"logica.gui.Bar",		
		"logica.gui.BarChart",
      		"logica.gui.MapElement",
   		"logica.gui.MapVisualization",
       		"logica.gui.RegionSquare",
         	"logica.gui.WorldMap",		
		"logica.gui.RegionBlob",
		"logica.gui.NorthNSouth",
		"logica.gui.NorthNSouth$Point2D",
		"logica.gui.NorthNSouth$Point3D",		
		"logica.gui.TxTButton",
		"logica.gui.TxTButtonMenu",				
		"logica.gui.TxTButtonMenu$ItemActionListener",
		"logica.gui.FlyingLabel",
   		"logica.gui.NullComponent",
		"logica.gui.LSlider",		
		"logica.gui.LMultiSlider",
  		"logica.gui.RegionControlPanel",
  		"logica.gui.InfoPanel",
  		"logica.gui.GraphPanel",				
           	"logica.gui.TimeSlider",
		"logica.gui.CloseButton",
		"logica.gui.MinimizeButton",
		"logica.gui.SubmitButton",
		"logica.gui.ResetButton",		
  		"logica.gui.Graph",			
	    	"logica.sim.Calc",
		"logica.sim.Simulation",
		"logica.sim.Vars",
		"logica.sim.Year",
		"logica.sim.io.SimInput",
		"logica.sim.io.SimOutput",
		"logica.sim.io.UIHandler",			
		"logica.sim.data.RegionData",
		"logica.sim.data.WorldData",
		"logica.sim.data.SimData",
		"logica.sim.data.uiData",
		"logica.sim.data.uiDataCount",
		"logica.utility.CSVReader",
		"logica.utility.Norm",
		"logica.utility.Utility",
		"logica.utility.Utility$Msg",		
		"logica.utility.GameSupport"		
        };
	
	String[] txt = {
			"loading classes .. ",
			"fetching data ..",
			"fetching status...",
			"fetching history...",
			"calculating..."
			};

	String sessionID;
	String userName;
	long refreshTime  = 0;

	private LogicaControlCenter gui;
	private Simulation sim;
	private Utility u;
	private GameSupport gameSupport;
	
	private int actYear;
	private int foresightYears = 5;

	private int playTime=0;
	private int userOnline=0;	
	
	private Thread th;
	private boolean running = false;
	private int status = 0;
	
	private Image offscreen;
	private Image logo;
	
	private float progress = 0;
	private String GAME_STATUS;
	private boolean offline = false;
	
	private int maxYear;
	private int calcYear = 0;	

	private int baseYear = 0;	
	

	public void init() 
	{	
		GAME_STATUS = getParameter("status");
		if(GAME_STATUS.equals("offline"))
		{
			offline = true;
			actYear = Integer.parseInt(getParameter("actYear"));
			foresightYears = Integer.parseInt(getParameter("foresightYears"));
			maxYear = actYear + foresightYears;
		}		
		else
		{
			offline = false;
			sessionID = getParameter("sid");
			userName = getParameter("userName");	
			refreshTime = Integer.parseInt(getParameter("refreshTime")) * 60 * 1000;	
			System.out.println("refreshTime: "+ refreshTime);
			baseYear=2000;
		}
			
		setBackground(Color.black);	
		// load Image from URL		
	}

	
	public void start() 
	{		
			
					
		if (th == null)
		{
			running = true;
			th = new Thread(this);
			th.start();

		}		
	}
	
	
	public void stop() 
	{
		if (th != null)
		{
			running = false;
			th = null;
		}			
		status = 0;
	}
	
	
	public void run() 
	{
		long t1,t2;
		
		t1 = System.currentTimeMillis();
			
		while (running)
		{
			// Load CLASSES
			if (status == 0)
			{	
				showStatus("./logicaland loading ..");
				repaint();

				try {	
					logo = getToolkit().getImage(new URL(getCodeBase() + "img/logicalogo.gif"));		

				} catch (Exception e) 
					{ System.out.println("error loading image: " + e.toString()); }
			
				// load Image as Resource
				//logo = getImageResource("logica/resources/lllogo.gif");	


				// load classes
	        		for (int i = 0; i < classArray.length; ++ i)
        			{
	            			try
        	    			{
			                	Class c = Class.forName(classArray[i]);			
						progress = (float) (i+1) / (float) (classArray.length + txt.length);
						
						//System.out.println("loaded " + classArray[i]);
						repaint();					
        	    			}
	           			 catch (Exception e)
        	    			{
        	        			System.out.println(e.toString());
	            			}
				}
				status++;
				repaint();
			
			}
			
			// Load SIMULATION and RegionPresets
			else if (status == 1)
			{												
				int[] uT = {};	
				u = new Utility(uT);		
				sim = new Simulation(getCodeBase().toString() + "data/", u);		
				progress += (float) 1 / (float) (classArray.length + txt.length);				
				status++;
				repaint();
			}
			
			// Load FLAGS
			else if (status == 2)
			{
				showStatus("./logicaland fetching status ..");
				if(!offline)			
				{				
					gameSupport = new GameSupport();
					gameSupport.readFlags(getCodeBase().toString() + "data_dyn/get_flags.php");
					actYear = gameSupport.getActYear();	
					foresightYears = gameSupport.getForeSight();
					playTime = gameSupport.getPlayTime();
					userOnline = gameSupport.getUserOnline();
					
					maxYear = actYear + foresightYears;					

					System.out.println("actYear: "+ actYear);
					System.out.println("future: "+ foresightYears);
						
				}
				progress += (float) 1 / (float) (classArray.length + txt.length);
				status++;			
				repaint();				
			}
			
			// Load UI history
			else if (status == 3)
			{
				if(!offline)			
				{
					showStatus("./logicaland fetching history ..");							
	
					try {
						sim.sUI.getUIHistory(new URL(getCodeBase().toString() + "data_dyn/get_zippedUIhistory.php"));
						sim.sUI.getUICount(new URL(getCodeBase().toString() + "data_dyn/get_UICount.php"));
					}
					catch (Exception e) 
					{
						System.out.println("fetching UIHistory failed: " + e.toString());
					}
				}

				progress = 0; //+= (float) 1 / (float) (classArray.length + txt.length);				
				
				
				status++;	
				calcYear = 0;	
				repaint();		
			}			
			
			// CALCULATE
			
			else if (status == 4)
			{
				showStatus("./logicaland calculating ..");				
				if (calcYear < maxYear)
				{
					sim.calculate(1);
					calcYear++;
					progress = (float) calcYear / (float) (maxYear);				
					repaint();
				}
				else status++;
				repaint();
			}
			
			else if (status == 5)
			{
				showStatus("./logicaland");				
				gui = new LogicaControlCenter(sim);
				if(!offline)			
				{				
					gui.setCodeBase(getCodeBase().toString());								
					gui.setSessionID(sessionID);
					gui.setUserName(userName);
				}
				gui.setLocation(0,0);
				//gui.setSize(800,520);

				
				gui.setForesight(foresightYears);								
				gui.setActYear(actYear);
				gui.setBaseYear(baseYear);
				gui.start();
				add(gui);
				repaint();
				status++;
			}
			
			// GAME LOOP
			if( !offline && refreshTime>0)			
			{
				t2 = System.currentTimeMillis();
			
				if (t2-t1 > refreshTime) 
				{
					System.out.println("refreshing flags");
					if (gameSupport.refreshFlags(getCodeBase().toString() + "data_dyn/refresh_flags.php?sid=" + sessionID))
					{
						
						if (actYear<gameSupport.getActYear()) 
						{
							gui.prepareNewYear();
							try {		
								sim.sUI.resetUI();
														
								//sim.sUI.setPresentYear(actYear);
								sim.sUI.getUIHistory(new URL(getCodeBase().toString() + "data_dyn/get_zippedUIhistory.php"));
								sim.sUI.getUICount(new URL(getCodeBase().toString() + "data_dyn/get_UICount.php"));												

								sim.reCalc(foresightYears+1);
								sim.calculate(gameSupport.getActYear()-actYear);								
								actYear = gameSupport.getActYear();
								gui.update2NewYear(actYear);

							}
							catch (Exception e) 
							{
								gui.showFailnewYears();							
								System.out.println("fetching UIHistory failed: " + e.toString());
							}									
											
						}
						else if (actYear > gameSupport.getActYear()) 
						{
							try {
								getAppletContext().showDocument(new URL(getCodeBase() + "?sid=" + sessionID));
							} catch(Exception e) {}
						}
							
						playTime = gameSupport.getPlayTime();
						userOnline = gameSupport.getUserOnline();
					}					
					repaint(); 
					
					
					t1 = System.currentTimeMillis();
				}
			}
			
			try {
				Thread.sleep(40);
			} catch (InterruptedException e) {}
		}
	}	

	
	public void update(Graphics g) 
	{
		if(offscreen == null) 
		{
			 offscreen = createImage(getSize().width, getSize().height);
		}	

		Graphics og = offscreen.getGraphics();
		paint(og);
		g.drawImage(offscreen, 0, 0, null);			
	}		
	
	
	
	public void paint(Graphics g) 
	{
		g.setColor(Color.black);
		g.fillRect(0,0,getSize().width,getSize().height);	

		g.setColor(Style.textLight);
		g.setFont( new Font("SansSerif",Font.PLAIN,10));		
		FontMetrics fm = getFontMetrics(Style.defaultFont);
		
		// draw Loading Bar
		if (status < txt.length)
		{ 						
			int txtPosX = (getSize().width - fm.stringWidth(txt[status]))/2;
			int txtPosY = (getSize().height - Style.defaultFontHeight)/2;
			
			if(logo!=null)
			{
				g.drawImage(logo, 
						(getSize().width - logo.getWidth(this))/2 , 
						(getSize().height - logo.getHeight(this))/2 - logo.getHeight(this) - 30,
						this);
			}
		
			g.drawString(txt[status], txtPosX, txtPosY);
			
			for (int i = 0; i<=50; i++)
			{
				if (i == (int) (progress*50.0F)) g.setColor(Style.textLight);
				if (i > (int) (progress*50.0F)) g.setColor(new Color(70,70,70));
				g.fillRect((getSize().width - 200)/2 + (4* i) , txtPosY + 10, 3, 3);
			}
						
		}
		else if (!offline)
		{		
			
			g.setColor(Style.textLight);		
			g.drawLine(0,Style.defaultFontHeight-2, 3, Style.defaultFontHeight-5);	
			g.drawLine(0,Style.defaultFontHeight-8, 3, Style.defaultFontHeight-5);				
					
			String txt_temp="LOGGED IN AS: ";		
			g.setColor(Style.textDark);		
			g.drawString(txt_temp, 20,Style.defaultFontHeight);			
			g.setColor(Color.white);
			g.drawString(userName, 20 + fm.stringWidth(txt_temp),Style.defaultFontHeight);

			txt_temp="USERS ONLINE: ";		
			g.setColor(Style.textDark);		
			g.drawString(txt_temp, 200,Style.defaultFontHeight);	
			g.setColor(Color.white);
			g.drawString("" +userOnline, 200 + fm.stringWidth(txt_temp),Style.defaultFontHeight);
				
			g.setColor(Style.defaultBorder);		
			g.drawLine(10, 2 +     Style.defaultFontHeight, 260 + fm.stringWidth(txt_temp), 2 +     Style.defaultFontHeight);	
			g.drawLine(10, 2 + 2 * Style.defaultFontHeight, 260 + fm.stringWidth(txt_temp), 2 + 2 * Style.defaultFontHeight);
			g.drawLine(260 + fm.stringWidth(txt_temp), 2 + 2 * Style.defaultFontHeight - 5, 260 + fm.stringWidth(txt_temp), 2+ 2 * Style.defaultFontHeight);
			g.drawLine(260 + fm.stringWidth(txt_temp), 2 +     Style.defaultFontHeight - 5, 260 + fm.stringWidth(txt_temp), 2+     Style.defaultFontHeight);
			g.drawLine(10, 0, 10, 2 + 2 * Style.defaultFontHeight);			
			g.drawLine(10, 0, 15, 0);
			
			txt_temp="YEAR: ";		
			g.setColor(Style.textDark);		
			g.drawString(txt_temp, 20, 2 * Style.defaultFontHeight);	
			g.setColor(Color.white);
			g.drawString(""+ (actYear+baseYear), 20 + fm.stringWidth(txt_temp), 2 * Style.defaultFontHeight);

			g.setColor(Style.darkLineColor);		
			
			
			g.setColor(Style.textDark);
			for (int i = 0; i<=50; i++)
			{
				if (i == playTime/2) g.setColor(Style.textLight);
				if (i == playTime/2+1) g.setColor(new Color(70,70,70));
				g.fillRect(fm.stringWidth(txt_temp) + 70 + (4* i) , 2 * Style.defaultFontHeight - 5, 3, 3);
			}
		}
		else
		{
			g.setColor(Style.textLight);		
			g.drawLine(0,Style.defaultFontHeight-2, 3, Style.defaultFontHeight-5);	
			g.drawLine(0,Style.defaultFontHeight-8, 3, Style.defaultFontHeight-5);				
					
			String txt_temp="LOGGED IN AS: ";		
			g.setColor(Style.textDark);		
			g.drawString(txt_temp, 20,Style.defaultFontHeight);			
			g.setColor(Color.white);
			g.drawString("<offline>", 20 + fm.stringWidth(txt_temp),Style.defaultFontHeight);
				
			g.setColor(Style.defaultBorder);		
			g.drawLine(10, 2 +     Style.defaultFontHeight, 260 + fm.stringWidth(txt_temp), 2 +     Style.defaultFontHeight);	
			g.drawLine(10, 2 + 2 * Style.defaultFontHeight, 260 + fm.stringWidth(txt_temp), 2 + 2 * Style.defaultFontHeight);
			g.drawLine(260 + fm.stringWidth(txt_temp), 2 + 2 * Style.defaultFontHeight - 5, 260 + fm.stringWidth(txt_temp), 2+ 2 * Style.defaultFontHeight);
			g.drawLine(260 + fm.stringWidth(txt_temp), 2 +     Style.defaultFontHeight - 5, 260 + fm.stringWidth(txt_temp), 2+     Style.defaultFontHeight);
			g.drawLine(10, 0, 10, 2 + 2 * Style.defaultFontHeight);			
			g.drawLine(10, 0, 15, 0);
			
			txt_temp="YEAR: ";		
			g.setColor(Style.textDark);		
			g.drawString(txt_temp, 20, 2 * Style.defaultFontHeight);	
			g.setColor(Color.white);
			g.drawString("" + actYear, 20 + fm.stringWidth(txt_temp), 2 * Style.defaultFontHeight);
		}
		
		super.paint(g);
	}	
	
	
	
	/* 
	// not functional for ns 4.x
	
	private Image getImageResource(String name)
	 {	
		Image img = null;
		try {
			URL url = getClass().getResource(name);
			img = getToolkit().getImage(url);
			MediaTracker mt = new MediaTracker(this);
			mt.addImage(img,0);
			mt.waitForAll();
		} catch (Exception e)    bla 
			{ System.out.println(e.toString()); }
		return img;
	 }	
	 
	 
	 
	 // fuer ns 4.x nur wenn als jar achrive
	 
	 private Image getImageResource(String name)
	 {	

		Image img = null;

		try {
		
			java.io.InputStream in = getClass().getResourceAsStream(name);

			if (in != null)
			{
				byte[] buffer = new byte[0];
				byte[] tmpbuffer = new byte[1024];
			
				while(true)
				{
					int len = in.read(tmpbuffer);
					if (len<= 0) break;
			
					byte[] newbuffer = new byte[buffer.length + len];
					System.arraycopy(buffer,0,newbuffer,0, buffer.length);
					System.arraycopy(tmpbuffer,0,newbuffer,buffer.length, len);			
					buffer = newbuffer;
				}
				img = Toolkit.getDefaultToolkit().createImage(buffer);
				in.close();
			}
		} catch (Exception e)  
			{ System.out.println("error loading image: " + e.toString()); }
		
		
		return img;
	}*/
		
}
