package logica.gui;


import java.util.*;
import java.awt.*;
import java.awt.event.*;

public class GraphPanel
extends Container
implements MouseMotionListener, MouseListener, ActionListener
{
			
	private float[] data2plot;
	private int[] coordArr;
		
	private int w;
	private int h;
	private String regionName = "";
	private String valueName = "";	
	private int actYear;
	
	private Graph graph;

	private NullComponent header;
	
	private boolean collapsed = false;
	private int margin = 2;	
	FontMetrics fm;
	
	private int headerH;

	private int graphH = 100;	
	private int graphW = 200;	
	
	//private CloseButton closeButton;
	private MinimizeButton minimizeButton;		
	
        
	public GraphPanel()
    	{		
		super();
		
		w = 200;
			
		setLayout(null);			
		setFont(Style.defaultFont);
		fm = getFontMetrics(Style.defaultFont);	
		headerH = Style.defaultFontHeight + 2*margin + 1;
		graphH = 100;			
					
		graph = new Graph(graphW,graphH);	
		graph.setLocation(0, headerH+2);
		add(graph);
		
		h = headerH + 2 + graphH;		
		setSize(w,h);
		
		minimizeButton = new MinimizeButton();
		minimizeButton.addActionListener(this);
		minimizeButton.setLocation(w - (minimizeButton.getWidth() +2),(headerH - minimizeButton.getHeight())/2);
		add(minimizeButton);	

		//closeButton = new CloseButton();
		//closeButton.addActionListener(this);							
		//closeButton.setLocation(w - (closeButton.getWidth() + 2),(headerH - closeButton.getHeight())/2);
		//add(closeButton);
		
		header = new NullComponent(w,headerH);
		header.addMouseMotionListener(this);
		header.addMouseListener(this);
		header.setLocation(0,0);
		add(header);														
	}
	
	
	
	public void paint(Graphics g)
	{			
		FontMetrics fm = getFontMetrics(Style.defaultFont);

		g.setColor(Color.black);
		g.fillRect(0, 0, w, h);		

		// HEADER
		
		g.setColor(Style.bgGreen1);
		g.fillRect(0, 0, w, headerH);		
		
		g.setColor(Style.defaultBorder);
		g.drawRect(0, 0, w-1, headerH);		
				
		g.setFont(Style.defaultFont);

		g.setColor(Color.white);		
		g.drawString(regionName,5,Style.defaultFontHeight);
		
		g.setColor(Style.textLight);
		g.drawString("| "+valueName,fm.stringWidth(regionName)+10,Style.defaultFontHeight);

		
		
		// GRAPH
		g.setColor(Color.black);
		g.fillRect(0,headerH + 2, w, graphH-1);

		int segs = 10;
		for (int i=0; i<segs; i++)
		{
			if (i % 2 == 0) g.setColor(Style.bgGreen1); else g.setColor(Style.bgGreen2);
			g.fillRect( i * (graphW/segs), headerH + 2, graphW/segs, graphH);				
		}
		
		g.setColor(Style.defaultBorder);		
		for (int i=0; i<segs; i++)
		{
			if (i % 2 == 1) 
			{
				g.drawLine(0, (headerH + 2) + i * (graphH/segs), 2, (headerH + 2) + i * (graphH/segs));				
				g.drawLine(graphW, (headerH + 2) + i * (graphH/segs), graphW - 3, (headerH + 2) + i * (graphH/segs));								
			}				
			else
			{
				g.drawLine(0, (headerH + 2) + i * (graphH/segs), 5, (headerH + 2) + i * (graphH/segs));							
				g.drawLine(graphW, (headerH + 2) + i * (graphH/segs), graphW - 6, (headerH + 2) + i * (graphH/segs));				
			}				
								
		}
		
				
		g.setColor(Style.defaultBorder);
		g.drawRect(0,headerH + 2, w-1,graphH-1);
		

		
		super.paint(g);
	}
		
	
	public void loadData(float[] values)
	{
		graph.loadData(values);		
	}
		
	public void toggleCollapse()
	{
		collapsed = !collapsed;
		if (collapsed) this.setSize(w,headerH + 2);
		else 
		{
			this.setSize(w,h);			
			Point Loc = this.getLocation();
			if ( Loc.y > getParent().getSize().height - h - 5)
			{
				this.setLocation(Loc.x, getParent().getSize().height - h - 5);
			}
		}
		repaint();		
	}

	
	public void setRegionName(String n)
	{
		if (n.length() > 10) regionName = n.substring(0,10) + "..";
		else regionName = n;
	}
	
	public void setValueName(String n)
	{
		valueName = n;
	}	
	
	public void setActYear(int yr)
	{
		graph.setActYear(yr);
	}	


	public int getWidth()
	{
		return w;
	}
	
	public int getHeight()
	{
		return h;
	}	
	
	
	// Mouse Listening
	
	
	private boolean dragging = false;
	private int oldX,oldY;
			
	public void mouseDragged(MouseEvent e) 
	{
		if (dragging)
		{
			Point oldLoc = getLocation();
			int xPos = Math.max(0, oldLoc.x + e.getX()-oldX);
			xPos = Math.min(xPos, getParent().getSize().width - w);
			int yPos = Math.max(0, oldLoc.y + e.getY()-oldY);
			yPos = Math.min(yPos, getParent().getSize().height - headerH -1);
			this.setLocation(xPos,yPos);
			repaint();
		}			
	}
	public void mousePressed(MouseEvent e) 
	{
		if (e.getY() < headerH)
		{
			oldX = e.getX();
			oldY = e.getY();
			dragging = true;
		}
	}
		
	public void mouseReleased(MouseEvent e) 
	{
		dragging = false;
	}
	
	public void mouseEntered(MouseEvent e) 	{}
		
	public void mouseExited(MouseEvent e) {}
		
	public void mouseMoved(MouseEvent e) {}

	public void mouseClicked(MouseEvent e) 
	{
		if (e.getClickCount() >1) toggleCollapse();
	}	
	
	
	public void actionPerformed(ActionEvent e)
	{
		String cmd = e.getActionCommand();
		if(cmd.equals("close"))		
		{
			this.setVisible(false);
			repaint();
		}
		if(cmd.equals("minimize"))		
		{
			this.toggleCollapse();
		}
	}				
			
}
    

    
    
     
     
     
     

