package logica.gui;

import logica.sim.io.*;

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import java.text.*;


public class InfoPanel
extends Container
implements MouseMotionListener, MouseListener, ActionListener
{
			
	private float[] data2plot;
	private int[] coordArr;
		
	private int w;
	private int h;
	
	private int actRegion;
		
	private String regionName = "";

	private String[] valueName = {"population", "area", "GNP", "food production"};
	private String[] simValueName = {"pop_stat", "area", "econ_gnp", "food_prod"};		
	private float[] value = {0.0F,0.0F,0.0F,0.0F};	

	private NullComponent header;
	
	private boolean collapsed = false;
	private int margin = 2;	
	FontMetrics fm;
	
	private int headerH;
	
	//private CloseButton closeButton;
	private MinimizeButton minimizeButton;
	
	private SimOutput simOut;		
	
        
	public InfoPanel()
    	{		
		super();
				
		w = 130;
			
		setLayout(null);			
		setFont(Style.defaultFont);		
		fm = getFontMetrics(Style.defaultFont);	
		headerH = Style.defaultFontHeight + 2*margin + 1;						
		
		h = headerH + 2 + 8 * Style.defaultFontHeight + 4;		
		
		setSize(w,h);
		
		minimizeButton = new MinimizeButton();
		minimizeButton.addActionListener(this);
		minimizeButton.setLocation(w - (minimizeButton.getWidth() +2),(headerH - minimizeButton.getHeight())/2);
		add(minimizeButton);	

		//closeButton = new CloseButton();
		//closeButton.addActionListener(this);							
		//closeButton.setLocation(w - (closeButton.getWidth() + 2),(headerH - closeButton.getHeight())/2);
		//add(closeButton);
		
		header = new NullComponent(w,headerH);
		header.addMouseMotionListener(this);
		header.addMouseListener(this);
		header.setLocation(0,0);
		
		add(header);														
	}
	
	
	
	public void paint(Graphics g)
	{			
		FontMetrics fm = getFontMetrics(Style.defaultFont);

		g.setColor(Color.black);
		g.fillRect(0, 0, w, h);		

		// header
		
		g.setColor(Style.bgGreen1);
		g.fillRect(0, 0, w, headerH);				
		g.setColor(Style.defaultBorder);
		g.drawRect(0, 0, w-1, headerH);						
		g.setFont(Style.defaultFont);
		g.setColor(Color.white);		

		g.drawString(regionName,5,Style.defaultFontHeight);		
		g.setColor(Style.textLight);
		g.drawString("| FACTS ",fm.stringWidth(regionName)+10,Style.defaultFontHeight);

		
		// info
		g.setColor(Color.black);
		g.fillRect(0,headerH + 2,w,h);				

		g.setColor(Style.bgGreen1);
		for (int i = 0; i<value.length; i++)					
			g.fillRect(1, headerH + 5 + (1 + i*2) * Style.defaultFontHeight, w, Style.defaultFontHeight);					

		g.setColor(Style.defaultBorder);
		g.drawRect(0,headerH + 2,w-1,h - (headerH+3));		

		
		g.setColor(Style.textDark);
		for (int i = 0; i<value.length; i++)
			g.drawString(valueName[i], 2, headerH + 2 + (1 + i*2) * Style.defaultFontHeight);

		NumberFormat nf = NumberFormat.getInstance();

		g.setColor(Color.white);				
		for (int i = 0; i<value.length; i++)
			g.drawString(nf.format(Math.round((double)value[i])), 2, headerH + 2 + (2 + i*2) * Style.defaultFontHeight);

			//! stupid workaround with explicit (double)-cast !
		
		super.paint(g);
	}
		

	/*
	public void loadData(int regionNr, int yr)
	{
		if (regionNr >= 0)
		{
			for (int i=0; i < value.length; i ++) 
				value[i] = simOut.getValue(simValueName[i],regionNr, yr);
			setRegionName(simOut.getName(regionNr));
		}
		else
		{
			for (int i=0; i < value.length; i ++) 
				value[i] = simOut.getValueWorld(simValueName[i],yr);
			setRegionName("WORLD");
		
		}
		repaint();
	}*/
	
	public void setValues(float[] vals)
	{
		value = null;
		value = vals;
	}
	
	public void setValueNames(String[] valNames)
	{
		valueName = null;
		valueName = valNames;
	}
	
		
	public void toggleCollapse()
	{
		collapsed = !collapsed;
		if (collapsed) this.setSize(w,headerH + 2);
		else 
		{
			this.setSize(w,h);			
			Point Loc = this.getLocation();
			if ( Loc.y > getParent().getSize().height - h - 5)
			{
				this.setLocation(Loc.x, getParent().getSize().height - h - 5);
			}
		}
		repaint();		
	}

	
	public void setRegionName(String n)
	{
		if (n.length() > 8) regionName = n.substring(0,8) + "..";
		else regionName = n;
	}
	

	public int getWidth()
	{
		return w;
	}
	
	public int getHeight()
	{
		return h;
	}	
	
	
	// Mouse Listening
	
	
	private boolean dragging = false;
	private int oldX,oldY;
			
	public void mouseDragged(MouseEvent e) 
	{
		if (dragging)
		{
			Point oldLoc = getLocation();
			int xPos = Math.max(0, oldLoc.x + e.getX()-oldX);
			xPos = Math.min(xPos, getParent().getSize().width - w - 0);
			int yPos = Math.max(0, oldLoc.y + e.getY()-oldY);
			yPos = Math.min(yPos, getParent().getSize().height - headerH - 1);			
			this.setLocation(xPos,yPos);
			repaint();
		}			
	}
	public void mousePressed(MouseEvent e) 
	{
		if (e.getY() < headerH)
		{
			oldX = e.getX();
			oldY = e.getY();
			dragging = true;
		}
	}
		
	public void mouseReleased(MouseEvent e) 
	{
		dragging = false;
	}
	
	public void mouseEntered(MouseEvent e) 	{}
		
	public void mouseExited(MouseEvent e) {}
		
	public void mouseMoved(MouseEvent e) {}

	public void mouseClicked(MouseEvent e) 
	{
		if (e.getClickCount() >1) toggleCollapse();
	}	
	
	
	public void actionPerformed(ActionEvent e)
	{
		String cmd = e.getActionCommand();
		if(cmd.equals("close"))		
		{
			this.setVisible(false);
			repaint();
		}
		if(cmd.equals("minimize"))		
		{
			this.toggleCollapse();
		}
	}				
			
}
    

    
    
     
     
     
     

