package logica.gui;

import logica.utility.*;

import java.awt.*;
import java.awt.event.*;

class LMultiSlider
extends Container
implements ActionListener
{		
	private int w = 200;
	private int h;
	
	private int margin = 6;
	
	private int bufferSize = 5;
	private int sliderNr = 0;
	
	private LSlider[] slider = new LSlider[bufferSize];
	private LSlider actualSlider;
	
	private ActionListener actionListener;
	
	public LMultiSlider()
	{
		super();
	}

	
	public void addSlider(String name)
	{
		addSlider(name,0.0F);
	}
		
	public void addSlider(String name, float value)
	{
		slider[sliderNr] = new LSlider(name, value);
		slider[sliderNr].addActionListener(this);
		sliderNr++;
		if (sliderNr >= bufferSize) 
		{
			bufferSize += 5;
			LSlider[] temp = new LSlider[bufferSize];
			System.arraycopy(slider,0,temp,0,slider.length);
			slider = temp;
		}	
		h = sliderNr * (slider[0].getHeight() + margin);
		arrange();
	}
	
	public void arrange()
	{
		removeAll();
		setLayout(null);
		setSize(w,h);
		
		for(int i=0; i<sliderNr; i++)
		{
			slider[i].setLocation(0, i * (slider[i].getHeight() + margin));
			add(slider[i]);
		}
	}
			
	public void paint(Graphics g)
	{
		g.setFont(Style.defaultFont);
		g.setColor(Style.textDark);
		for(int i=0; i<sliderNr; i++)
		{
			g.drawString(slider[i].getName(), slider[i].getWidth() + 10, 8 + i * (slider[i].getHeight() + margin));
		}		
		super.paint(g);
	}
	

	public void setValues(float[] values)
	{
		for(int i=0; i<sliderNr; i++) slider[i].setValue(values[i]);
	}
	
	public void setSecondValues(float[] values)
	{
		for(int i=0; i<sliderNr; i++) slider[i].setSecondValue(values[i]);
	}
	

	
	public float[] getValues()
	{
		float[] values = new float[sliderNr];
		for(int i=0; i<sliderNr; i++) values[i] = slider[i].getValue();
		return values;
	}
	
	

	public void distributeValueChange()
	{
		float[] value = new float[sliderNr];
		for(int i=0;i<sliderNr; i++) value[i] = slider[i].getValue();

		float[] n_value = Norm.normalizeToSum(value);
		for(int i=0;i<sliderNr; i++) slider[i].setValue(n_value[i]);
	}
	
	public LSlider getActual()
	{
		return actualSlider;
	}	
	
	
	public void actionPerformed(ActionEvent e) 
	{
		String cmd = e.getActionCommand();
		if (cmd.equals("value_change"))
		{
			actualSlider = (LSlider) e.getSource();
			if(actionListener != null) 
			{
				actionListener.actionPerformed(new ActionEvent(this, ActionEvent.ACTION_PERFORMED, "dist_value_change"));				
			}				
			distributeValueChange();
		}
		else if(cmd.equals("change_done"))
		{
			actualSlider = (LSlider) e.getSource();
			if(actionListener != null) 
			{
				actionListener.actionPerformed(new ActionEvent(this, ActionEvent.ACTION_PERFORMED, "dist_change_done"));				
			}				
		}		
			
	}	
	
	
	public void addActionListener(ActionListener listener)
	{
		actionListener = AWTEventMulticaster.add(actionListener, listener);
	}
		
	public void removeActionListener(ActionListener listener)
	{
		actionListener = AWTEventMulticaster.remove(actionListener, listener);
	}					
}
	
	
	
