package logica.gui;

import java.awt.*;
import java.awt.event.*;


public class LSlider
extends Component
implements MouseMotionListener, MouseListener
{
	private String name;
	private int id;	
		
	private int w;
	private int h = 8;	

	private float value, secondValue;
	private int pos, secondPos;
	
	private boolean hasFocus;
	private boolean isActive;	
	
	private ActionListener actionListener;

	
	public LSlider(String name)
	{
		this(name, 100, 0.5F);
	}
	
	public LSlider(String name, float value)
	{
		this(name, 100, value);
	}
	
	public LSlider(String name,  int width, float value)
	{
		super();
        
		this.name = name;
		this.w = width;
		this.value = value;
		this.pos = (int) (value * (float) width);
		
		hasFocus = false;
		isActive = false;				
		
		setSize(w+2,h);				
		
		addMouseMotionListener(this);		
		addMouseListener(this);
	}
        

	public int getId()
	{
		return id;
	}	
	
	
	public String getName()
	{
		return name;
	}	
	
	public int getPos()
	{
		return pos;
	}			
		
	
	public void setValue(float v)
	{
		value = v;
		pos = (int) (value * (float) w);
		repaint();
	}
	
	public void setSecondValue(float v)
	{
		secondValue = v;
		secondPos = (int) (secondValue * (float) w);
		repaint();
	}		
	
	public float getValue()
	{
		value = (float) pos / (float) w;
		return value;
	}	
	
	
	public Dimension getPreferredSize()
	{
		return new Dimension(w+2,h);
	}
	

	public Dimension getMinimumSize()
	{
		return new Dimension(w+2,h);
	}
	
	public int getWidth()
	{
		return w+2;
	}
	
	public int getHeight()
	{
		return h;
	}
	
	
	public boolean isFocusTraversable()
	{
		return true;
	}	

	
	public void paint(Graphics g)
	{
			
		if (isActive)
		{
			g.setColor(Style.sliderColor);
			g.drawRect(0, h/2 - 1, w-1, 1);

			g.setColor(Style.MiddleGrey);
			g.fillRect(secondPos, 0, 2, h);

			g.setColor(Style.textLight);
			g.fillRect(pos - 1, 0, 3, h);			
			//g.fillRect(pos + 1, 0, 3, h);	

		}
		else 
		{
			g.setColor(Style.sliderColor);
			g.drawRect(0, h/2 - 1, w-1, 1);

			g.setColor(Style.MiddleGrey);
			g.fillRect(secondPos, 0, 2, h);

			g.setColor(Style.textDark);
			g.fillRect(pos - 1, 0, 3, h);			
			//g.fillRect(pos + 1, 0, 1, h);									
		}		 
	}

	public void addActionListener(ActionListener listener)
	{
		actionListener = AWTEventMulticaster.add(actionListener, listener);
	}
		
	public void removeActionListener(ActionListener listener)
	{
		actionListener = AWTEventMulticaster.remove(actionListener, listener);
	}		
	

	public void mouseDragged(MouseEvent e)
	{
		isActive = true;
		int mouseX = e.getX();
		pos = mouseX;
		pos = Math.max(0,mouseX);
		pos = Math.min(w,pos);
		if(actionListener != null) 
		{
				actionListener.actionPerformed(new ActionEvent(this, ActionEvent.ACTION_PERFORMED, "value_change"));				
		}				
		repaint();
	}		

	public void mouseMoved(MouseEvent e)
	{
	}		
	
	public void mouseClicked(MouseEvent e) 
	{
	}
	
	public void mouseEntered(MouseEvent e) 
	{
		requestFocus();
	}
	
	public void mouseExited(MouseEvent e) 
	{
		isActive = false;
		repaint();
	}
	
	public void mousePressed(MouseEvent e) 
	{
		int mouseX = e.getX();
		pos = mouseX;
		pos = Math.max(0,mouseX);
		pos = Math.min(w,pos);
		isActive = true;
		if(actionListener != null) 
		{
				actionListener.actionPerformed(new ActionEvent(this, ActionEvent.ACTION_PERFORMED, "value_change"));				
		}						
		repaint();	
	}
	
	public void mouseReleased(MouseEvent e) 
	{
		isActive = false;
		if(actionListener != null) 
		{
				actionListener.actionPerformed(new ActionEvent(this, ActionEvent.ACTION_PERFORMED, "change_done"));				
		}					
		repaint();		

	}
	
}
	 




