package logica.gui;

import logica.utility.*;
import logica.sim.*;
import logica.sim.io.*;

import java.awt.*;
import java.awt.event.*;
import java.net.*;


public class LogicaControlCenter 
extends Container 
implements ActionListener
{
	private SimOutput dataSrc;
	private Simulation sim;


	private MapVisualization map;
	private RegionControlPanel controlPanel;
	private GraphPanel graphPanel;
	private InfoPanel infoPanel;	

	
	private TxTButton butt1;
	private TxTButton butt2;
	private TxTButton butt3;		//SEPPCODE	
	private SubmitButton submitButton;
	private ResetButton resetButton;	

	private Dialog dialog;

	private TxTButtonMenu menu;	
	private TimeSlider timeslider;
	private FlyingLabel label;
			
	private String actValue = "area";		


	private int actYear = 50;
	private int foresightYears = 5;
	private int displayYear = actYear;
	
	private int actRegion = -1;	

	private String sessionID;		
	private String codebase;
	private String userName;	
				
				
	private int menuH = 0;
	private int buttonH = 0;


	private int baseYear = 0;	

	
	private int regionNr;
	
	public LogicaControlCenter(Simulation sim)
	{
		super();
		this.sim = sim;
		dataSrc = sim.sOut;
		regionNr = sim.getNumRegs();						
	}
	 
	
	public void start()
	{	
	
		displayYear = actYear;
		actValue="area";
		
		setBackground(Color.black);
		setLayout(null);
		
		setSize(800,550);
		
		dialog = new Dialog(800,550);
		dialog.setVisible(false);
		add(dialog);
		
		
		label = new FlyingLabel("nothing selected ");
		add(label);
		label.setVisible(false);
		
		controlPanel = new RegionControlPanel(sim);
		controlPanel.addActionListener(this);
		controlPanel.setLocation((getSize().width - controlPanel.getWidth())/2, (getSize().height - controlPanel.getHeight())/2);
		controlPanel.setVisible(false);
		controlPanel.setActYear(actYear);
		add(controlPanel);
		
		graphPanel = new GraphPanel();
		graphPanel.setLocation(800 - graphPanel.getWidth(),5);
		graphPanel.loadData(dataSrc.getValueWorldTime(actValue,0,actYear+foresightYears));
		graphPanel.setActYear(actYear);
		graphPanel.setValueName(actValue);
		graphPanel.setRegionName("WORLD");		
		add(graphPanel);			
		
		infoPanel = new InfoPanel();
		infoPanel.setLocation(10,40);
		String[] infoValNames = {"population", "area", "GNP", "food production"};
		infoPanel.setValueNames(infoValNames);			
		float[] infoValues = { dataSrc.getValueWorld("pop_stat", displayYear), 
					dataSrc.getValueWorld("area", displayYear), 
					dataSrc.getValueWorld("econ_gnp", displayYear), 
					dataSrc.getValueWorld("food_prod", displayYear)};	
		infoPanel.setValues(infoValues);
		infoPanel.setRegionName("WORLD");
		add(infoPanel);	

		
		menu = new TxTButtonMenu();
		menu.addItem("area");
		menu.addItem("population");
		menu.addItem("gnp");	
		menu.addItem("income per cap");				
		menu.addItem("tni");
		menu.addItem("production");				
		menu.addItem("technology");
		menu.addItem("oil consum");		
		menu.addItem("education");			
		menu.addItem("nutrition level");
		
		//menu.addItem("util_avail");					
	
		menu.setLocation(11, 455 - menu.getHeight());	
		add(menu);
		menu.addActionListener(this);
		
		butt1 = new TxTButton("WORLDMAP");
		butt1.setLocation(10, 455);
		add(butt1);
		butt1.addActionListener(this);

		butt2 = new TxTButton("BAR CHART");
		butt2.setLocation(10 + butt1.getWidth() + 10, 455);
		add(butt2);
		butt2.addActionListener(this);
		
		butt3 = new TxTButton("NORTH/SOUTH");
		butt3.setLocation(10 + (butt1.getWidth() + 10) * 2, 455);
		add(butt3);
		butt3.addActionListener(this);

		if (sessionID!=null)
		{
			submitButton = new SubmitButton();
			submitButton.setLocation(getSize().width - submitButton.getWidth() -10, 516);	
			add(submitButton);
			submitButton.addActionListener(this);
			
			resetButton = new ResetButton();
			resetButton.setWidth(submitButton.getWidth());
			resetButton.setLocation(getSize().width - submitButton.getWidth() -10, 509 - resetButton.getHeight());	
			add(resetButton);
			resetButton.addActionListener(this);			
		}

		
		timeslider = new TimeSlider(actYear,actYear + foresightYears);
		timeslider.setLocation(0, 495);
		add(timeslider);
		timeslider.addActionListener(this);	
		
			
		map = new WorldMap(800,400);
		initMap();
		butt1.setActive(true);
		
	}
	
	
	public void paint(Graphics g)
	{				
		g.setColor(Style.defaultBorder);
		//g.drawLine(0,49,getSize().width,49);

		//g.drawLine(20, 455, 20 + 3* (butt1.getWidth()+10) + 10,455);		
		//g.drawLine( 0, 455, getSize().width, 455);					
		g.drawLine(10, 455 + butt1.getHeight() - 1, 10 + 3* (butt1.getWidth()+10) + 10, 455 + butt1.getHeight()-1);			
		g.drawLine(10, 455, 10 + 3* (butt1.getWidth()+10) + 10, 455);	
		
		g.drawLine(10, 455 - menu.getHeight(), 10, 455 + butt1.getHeight() -1);
		g.drawLine(10, 455 - menu.getHeight(), 15, 455 - menu.getHeight());
				
		g.drawLine(10 + 3* (butt1.getWidth()+10) + 10, 455, 10 + 3* (butt1.getWidth()+10) + 10, 455 + butt1.getHeight()-1);							
	
		g.drawLine(10, 512, getSize().width, 512);		
		
		g.drawLine(10, 494, 10, 512);
		g.drawLine(10, 494, 15, 494);
		

		g.setColor(Style.textLight);		

		g.drawLine(0,455 - menu.getHeight() + 15 -2, 3, 455 - menu.getHeight() + 15 - 5);	
		g.drawLine(0,455 - menu.getHeight() + 15 -8, 3, 455 - menu.getHeight() + 15 - 5);

		g.drawLine(0,460 + butt1.getHeight()/2 -2, 3, 460 + butt1.getHeight() /2 -5);	
		g.drawLine(0,460 + butt1.getHeight()/2 -8, 3, 460 + butt1.getHeight() /2 -5);

		if (sessionID!=null)
		{
			g.drawLine(getSize().width - submitButton.getWidth() - 20, 516 + submitButton.getHeight()/2 - 3 , getSize().width - submitButton.getWidth() - 17, 516 + submitButton.getHeight()/2);
			g.drawLine(getSize().width - submitButton.getWidth() - 20, 516 + submitButton.getHeight()/2 + 3 , getSize().width - submitButton.getWidth() - 17, 516 + submitButton.getHeight()/2);		
		}
				
		super.paint(g);
	}			

	
	public void initMap()
	{
		map.addActionListener(this);
		map.setLocation(0,55);
		
		for (int i=0; i<regionNr; i++)
		{
			map.addRegion(dataSrc.getName(i), i, dataSrc.getPosX(i), dataSrc.getPosY(i), dataSrc.getValue(actValue,i, displayYear));
		}			

		map.reArrange();
		add(map);	
	}	
		
		
	
	public void actionPerformed(ActionEvent e) 
	{
		String cmd = e.getActionCommand();
		

		if(cmd.equals("edit"))
		{
			MapElement src = (MapElement) e.getSource();			
			actRegion = src.getId();
			
			map.activate(actRegion);
			
			reloadInfoPanel();
			reloadGraph();			
			
			controlPanel.loadData(actRegion,actYear);			
			controlPanel.setVisible(true);
						
			repaint();
		}
		else if(cmd.equals("deselect"))
		{		
			actRegion = -1;
			map.activate(actRegion);
		
			controlPanel.setVisible(false);	
			
			reloadInfoPanel();														
			reloadGraph();

			repaint();
		}		
		else if(cmd.equals("BAR CHART"))
		{
			remove(map);
			map = null;
			map = new BarChart(800,400);
			initMap();
			map.activate(actRegion);
			
			((TxTButton)e.getSource()).setActive(true);
			butt1.setActive(false);
			butt3.setActive(false);			
			
			repaint();
		}
		else if(cmd.equals("WORLDMAP"))
		{
			remove(map);
			map = null;
			map = new WorldMap(800,400);
			initMap();
			map.activate(actRegion);
			
			((TxTButton)e.getSource()).setActive(true);
			butt2.setActive(false);		
			butt3.setActive(false);			
			
			repaint();
		}
		else if(cmd.equals("NORTH/SOUTH"))
		{
			remove(map);
			map = null;
			map = new NorthNSouth(800,400);
			initMap();
			map.activate(actRegion);
			
			((TxTButton)e.getSource()).setActive(true);
			butt1.setActive(false);
			butt2.setActive(false);
			
			repaint();
		}
		else if(cmd.equals("time_change"))
		{
			TimeSlider source = (TimeSlider)e.getSource() ;
			
			displayYear = source.getYear();
			map.setNewData(dataSrc.getValueRegs(actValue,displayYear));
			//controlPanel.loadData(actRegion,displayYear);	
			
			label.setText("year: " + (baseYear + displayYear));			
			Point newPos = new Point(source.getLocation());
			newPos.translate(source.getPos() - label.getWidth() / 2, - label.getHeight() + 1);	
			
			newPos.x = Math.max(1,newPos.x);
			newPos.y = Math.max(1,newPos.y);
			newPos.x = Math.min(getSize().width  - label.getWidth() - 1,newPos.x);
			newPos.y = Math.min(getSize().height - label.getHeight() -1 ,newPos.y);			
			
			label.setLocation(newPos);
			label.setVisible(true);		
			
			reloadInfoPanel();
			
			repaint();
		}
		else if(cmd.equals("user_change_done"))
		{
			sim.reCalc(foresightYears+1);
			map.setNewData(dataSrc.getValueRegs(actValue,displayYear));

			controlPanel.loadData(actRegion,actYear);

			reloadInfoPanel();									
			reloadGraph();									
						
			repaint();
		}			
		else if(cmd.equals("change_done"))
		{
			label.setVisible(false);								
			repaint();
		}			
		else if(cmd.equals("SUBMIT"))
		{
			try {
				dialog.setText("submitting..");
				dialog.setVisible(true);
				String return_string = sim.sUI.sendToServer(new URL(codebase + "data_dyn/submit_UI.php"), sessionID, actYear);
				dialog.setText(return_string);
				if (return_string.equals("submit done.")) 
				{
					if (userName.equals("bigguest")) 
					{
						dialog.setText(return_string + ".. reseting input");
						sim.sUI.resetUI();
						sim.reCalc(foresightYears+1);			
						map.setNewData(dataSrc.getValueRegs(actValue,displayYear));			
						reloadInfoPanel();									
						reloadGraph();	
						if (actRegion >= 0) controlPanel.loadData(actRegion,actYear);								
						repaint();						
					}
					else submitButton.setValid(false); 					
				}							
				dialog.waitForOK();				
				
			} catch (Exception ex) 
			{ System.out.println("error submitting " +  ex.toString()); }
		}	
		else if(cmd.equals("RESET"))
		{
			sim.sUI.resetUI();
			sim.reCalc(foresightYears+1);
			
			map.setNewData(dataSrc.getValueRegs(actValue,displayYear));
			
			reloadInfoPanel();									
			reloadGraph();																
			
			if (actRegion >= 0) controlPanel.loadData(actRegion,actYear);
			repaint();
		}							
		else // Display Value Change
		{
			actValue = cmd;
			map.setNewData(dataSrc.getValueRegs(actValue,displayYear));

			reloadInfoPanel();									
			reloadGraph();												

			repaint();
		}
	}	
	
	public void reloadInfoPanel()
	{
			if (actRegion >= 0)
			{				
				float[] infoValues = { dataSrc.getValue("pop_stat",actRegion, displayYear), 
					dataSrc.getValue("area",actRegion, displayYear), 
					dataSrc.getValue("econ_gnp",actRegion, displayYear), 
					dataSrc.getValue("food_prod",actRegion, displayYear)};			
				infoPanel.setValues(infoValues);
				infoPanel.setRegionName(dataSrc.getName(actRegion));								
				
			}
			else
			{
				float[] infoValues = { dataSrc.getValueWorld("pop_stat", displayYear), 
					dataSrc.getValueWorld("area", displayYear), 
					dataSrc.getValueWorld("econ_gnp", displayYear), 
					dataSrc.getValueWorld("food_prod", displayYear)};	
				infoPanel.setValues(infoValues);								
				infoPanel.setRegionName("WORLD");				
			}	
	}
	
	public void reloadGraph()
	{
			if (actRegion >= 0)
			{
				graphPanel.loadData(dataSrc.getValueTime(actValue,actRegion,0,actYear+foresightYears));			
				graphPanel.setValueName(actValue);
				graphPanel.setRegionName(dataSrc.getName(actRegion));				
			}
			else
			{
				graphPanel.loadData(dataSrc.getValueWorldTime(actValue,0,actYear+foresightYears));			
				graphPanel.setValueName(actValue);
				graphPanel.setRegionName("WORLD");								
			}	
	}		
	
	public void prepareNewYear()
	{
		controlPanel.setVisible(false);	
		
		dialog.showOK(false);
		dialog.setVisible(true);			
		dialog.setText("year changed. updating data..");
		repaint();
								 				
		repaint();	
			
	}	
	
	public void update2NewYear(int yr)
	{						
		actYear = yr;	
		
		timeslider.setActYear(actYear);
		timeslider.setMaxYear(actYear+foresightYears);
		
		graphPanel.setActYear(actYear);	
		reloadGraph();	
		
		submitButton.setValid(true);
		dialog.setText("year changed. updating data.. done");
		dialog.showOK(true);			 				
		repaint();				
	}
	
	public void showFailnewYears()
	{
		dialog.setText("updating data.. failed !");
		dialog.showOK(true);			 				
		repaint();				
	}		
	
	
	public void setCodeBase(String baseUrl)
	{
		codebase = baseUrl;
	}
	
	public void setUserName(String name)
	{
		userName = name;
	}	

	public void setSessionID(String sid)
	{
		sessionID = sid;
	}

	public void setForesight(int yr)
	{
		foresightYears = yr;
	}
	
	public void setActYear(int yr)
	{
		actYear = yr;
	}
	
	public void setBaseYear(int yr)
	{
		baseYear = yr;
	}				
	

}
