package logica.gui;

import logica.utility.*;
import logica.sim.*;
import logica.sim.io.*;

import java.awt.*;
import java.awt.event.*;
import java.net.*;


public class LogicaStatisticCenter 
extends Container 
implements ActionListener
{
	private MapVisualization map;
	private InfoPanel infoPanel;	
	
	private TxTButton butt1;
	private TxTButton butt2;
	private TxTButton butt3;	
	private TxTButtonMenu menu;	
	private FlyingLabel label;
			
	private int actValue = 3;		
	private int actRegion = -1;				
				
	private int menuH = 0;
	private int buttonH = 0;
	
	private CSVReader dataSrc;

	
	private int regionNr;
	
	public LogicaStatisticCenter(CSVReader data_in)
	{
		super();
		dataSrc = data_in;
		regionNr = dataSrc.countRows();						
	}
	  
	 
	
	public void start()
	{	
		
		setBackground(Color.black);
		setLayout(null);
		
		setSize(800,550);
	
		
		label = new FlyingLabel("nothing selected ");
		add(label);
		label.setVisible(false);
		
		
		infoPanel = new InfoPanel();
		infoPanel.setLocation(5,20);
		String[] infoValNames = {"population", "internet users", "./logica_LOGINS", "./logica_SUBMITS"};
		infoPanel.setValueNames(infoValNames);									
		reloadInfoPanel();
		add(infoPanel);	

		
		menu = new TxTButtonMenu();
		menu.addItem("area");
		menu.addItem("population");	
		menu.addItem("internet users");	
		menu.addItem("internet isp");
		menu.addItem("logica_LOGINS");	
		menu.addItem("logica_SUBMITS");		
		menu.setLocation(11, 455 - menu.getHeight());	
		add(menu);
		menu.addActionListener(this);
		
		butt1 = new TxTButton("WORLDMAP");
		butt1.setLocation(10, 455);
		add(butt1);
		butt1.addActionListener(this);

		butt2 = new TxTButton("BAR CHART");
		butt2.setLocation(10 + butt1.getWidth() + 10, 455);
		add(butt2);
		butt2.addActionListener(this);
		
		butt3 = new TxTButton("NORTH/SOUTH");
		butt3.setLocation(10 + (butt1.getWidth() + 10) * 2, 455);
		add(butt3);
		butt3.addActionListener(this);
		
		map = new WorldMap(800,400);
		initMap();
		butt1.setActive(true);
		
	}
	
	
	public void paint(Graphics g)
	{				
		g.setColor(Style.defaultBorder);
				
		g.drawLine(10, 455 + butt1.getHeight() - 1, 10 + 3* (butt1.getWidth()+10) + 10, 455 + butt1.getHeight()-1);
		g.drawLine(10, 455, 10 + 3* (butt1.getWidth()+10) + 10, 455);	
		
		g.drawLine(10, 455 - menu.getHeight(), 10, 455 + butt1.getHeight() -1);
		g.drawLine(10, 455 - menu.getHeight(), 15, 455 - menu.getHeight());
				
		g.drawLine(10 + 3* (butt1.getWidth()+10) + 10, 455, 10 + 3* (butt1.getWidth()+10) + 10, 455 + butt1.getHeight()-1);
	
		/*
		g.drawLine(10, 512, 500, 512);		
		g.drawLine(10, 494, 10, 512);
		g.drawLine(10, 494, 15, 494);
		*/
		
		g.setFont(Style.defaultFont);
		g.drawString("data source: all dataset (except logica actions) taken from: CIA world factbook, 2001",10, 505);
		

		g.setColor(Style.textLight);		

		/*
		g.drawLine(0,455 - menu.getHeight() + 15 -2, 3, 455 - menu.getHeight() + 15 - 5);	
		g.drawLine(0,455 - menu.getHeight() + 15 -8, 3, 455 - menu.getHeight() + 15 - 5);

		g.drawLine(0,460 + butt1.getHeight()/2 -2, 3, 460 + butt1.getHeight() /2 -5);	
		g.drawLine(0,460 + butt1.getHeight()/2 -8, 3, 460 + butt1.getHeight() /2 -5);
		*/
				
		super.paint(g);
	}

	
	public void initMap()
	{
		map.addActionListener(this);
		map.setLocation(0,55);
		
		int i=0;
		dataSrc.rewind();
		
		while(dataSrc.hasNextRow())
		{
			
			String[] row = dataSrc.fetchRow();			
			map.addRegion(
				row[0],
				i,
				Float.valueOf(row[1]).floatValue(), 
				Float.valueOf(row[2]).floatValue(), 
				Float.valueOf(row[actValue]).floatValue()
				);
			i++;
		}			
		
		
		map.reArrange();
		add(map);	
	}	
	
	public void reloadInfoPanel()
	{	
		int[] colnbrs = {4,6,8,9};
		
		if (actRegion >= 0)
		{						
			String[] row  = dataSrc.fetchRow(actRegion);
			float[] infoValues = { Float.valueOf(row[colnbrs[0]]).floatValue(),
						Float.valueOf(row[colnbrs[1]]).floatValue(),
						Float.valueOf(row[colnbrs[2]]).floatValue(),
						Float.valueOf(row[colnbrs[3]]).floatValue() };	
						
			infoPanel.setValues(infoValues);
			infoPanel.setRegionName(row[0]);								
			
		}
		else
		{
			float[] infoValues = new float[4];
			
			for(int j=0; j<colnbrs.length; j++)
			{
				String[] col = dataSrc.fetchColumn(colnbrs[j]);
				
				infoValues[j] = 0;
				for (int i = 0; i < dataSrc.countRows(); i++)
				{
					infoValues[j] += Float.valueOf(col[i]).floatValue();
				}
			}
			
			infoPanel.setValues(infoValues);								
			infoPanel.setRegionName("WORLD");				
		}	
	}	
	
	
	public void actionPerformed(ActionEvent e) 
	{
		String cmd = e.getActionCommand();
		

		if(cmd.equals("edit"))
		{
			MapElement src = (MapElement) e.getSource();
			
			actRegion = src.getId();
			
			map.activate(actRegion);

			reloadInfoPanel();								

			repaint();
		}
		else if(cmd.equals("deselect"))
		{		
			actRegion = -1;
			map.activate(actRegion);
						
			reloadInfoPanel();	
			
			repaint();
		}		
		else if(cmd.equals("BAR CHART"))
		{
			remove(map);
			map = null;
			map = new BarChart(800,400);
			initMap();
			map.activate(actRegion);
			
			((TxTButton)e.getSource()).setActive(true);
			butt1.setActive(false);
			butt3.setActive(false);			
			
			repaint();
		}
		else if(cmd.equals("WORLDMAP"))
		{
			remove(map);
			map = null;
			map = new WorldMap(800,400);
			initMap();
			map.activate(actRegion);
			
			((TxTButton)e.getSource()).setActive(true);
			butt2.setActive(false);		
			butt3.setActive(false);			
			
			repaint();
		}
		else if(cmd.equals("NORTH/SOUTH"))
		{
			remove(map);
			map = null;
			map = new NorthNSouth(800,400);
			initMap();
			map.activate(actRegion);
			
			((TxTButton)e.getSource()).setActive(true);
			butt1.setActive(false);
			butt2.setActive(false);
			
			repaint();
		}
				
		else // Display Value Change
		{
			if (cmd.equals("area")) actValue=3;
			else if (cmd.equals("population")) actValue=4;
			else if (cmd.equals("gdp_per_cap")) actValue=5;
			else if (cmd.equals("internet users")) actValue=6;
			else if (cmd.equals("internet isp")) actValue=7;
			else if (cmd.equals("logica_LOGINS")) actValue=8;
			else if (cmd.equals("logica_SUBMITS")) actValue=9;			
														
			String[] tmp = dataSrc.fetchColumn(actValue);
			float[] data = new float[dataSrc.countRows()];
			for (int i=0; i< dataSrc.countRows(); i++) data[i] = Float.valueOf(tmp[i]).floatValue();
				
			map.setNewData(data);
		
			//infoPanel.loadData(actRegion, displayYear);					

			repaint();
		}
	}		

}
