package logica.gui;

import java.awt.*;
import java.awt.event.*;
import java.text.*;

public abstract class MapVisualization 
extends Container
implements MouseListener
{
	int buffersize = 300;	
		
	int w;
	int h;
	
	int nrOfRegions;	
	
	FlyingLabel regionLabel;
	ActionListener actionListener;
	
	int active;
        
    public MapVisualization(int width, int height)
    {		
		super();
		
		this.w = width;
		this.h = height;
				
		nrOfRegions = 0;				
		regionLabel = new FlyingLabel("no country selected");
		regionLabel.setVisible(false);
		addMouseListener(this);
	}
	
	
    
	public abstract void addRegion(String name, int id, float x, float y, float value);
	public abstract void setNewData(float[] values);
	public abstract void activate(int id);

	abstract void arrange();
	abstract void scaleToFit(MapElement[] object, float min, float max);		
	
		
	public void paint(Graphics g)
	{	
		super.paint(g);
	}	
		
	public Dimension getPreferredSize()
	{
		return new Dimension(w,h);
	}	

	public Dimension getMinimumSize()
	{
		return new Dimension(w,h);
	}		
	
	public void removeAllRegions()
	{
		nrOfRegions = 0;
	}	
	
		
	public void reArrange()
	{
		arrange();
	}
	
	
	public int getId()
	{
		return -1;
	}
	

	public void sortByValue(MapElement[] objects)
	{
		boolean sorted;
		do 
		{
			sorted = true;
			for (int i = 0; i < nrOfRegions - 1; ++i) 
			{
				if (objects[i].getValue() > objects[i+1].getValue()) 
				{
					MapElement tmp = objects[i];
					objects[i] = objects[i + 1];
					objects[i + 1] = tmp;
					sorted = false;
				}
			}
		} 
		while (!sorted);
	}
	
	
	public void sortById(MapElement[] objects)
	{
		boolean sorted;
		do 
		{
			sorted = true;
			for (int i = 0; i < nrOfRegions - 1; ++i) 
			{
				if (objects[i].getId() > objects[i+1].getId()) 
				{
					MapElement tmp = objects[i];
					objects[i] = objects[i + 1];
					objects[i + 1] = tmp;
					sorted = false;
				}
			}
		} 
		while (!sorted);
	}	

	
	
	public void mouseEntered(MouseEvent e)
	{	
		NumberFormat nf = NumberFormat.getInstance();
		String float2Txt;
		
		if (e.getSource()!=this)
		{
			MapElement source = (MapElement)e.getSource() ;
//			regionLabel.setText(source.getName());
			float2Txt = nf.format((double)source.getAbsValue());
			regionLabel.setText((source.getName() + " | " + float2Txt));
			
			Point newPos = new Point(source.getLocation());
			newPos.translate(e.getX() - 10 ,e.getY() - regionLabel.getHeight() - 10);
			
			newPos.x = Math.max(5,newPos.x);
			newPos.y = Math.max(5,newPos.y);
			
			newPos.x = Math.min(w  - regionLabel.getWidth() - 5 ,newPos.x);
			newPos.y = Math.min(h - regionLabel.getHeight() - 5 ,newPos.y);
			
			regionLabel.setLocation(newPos);
			regionLabel.setVisible(true);	
		}
	}	
		
	public void mouseReleased(MouseEvent e)
	{

		if(actionListener != null) 
		{
			if (e.getSource()==this) actionListener.actionPerformed(new ActionEvent(e.getSource(), ActionEvent.ACTION_PERFORMED, "deselect"));				
			else actionListener.actionPerformed(new ActionEvent(e.getSource(), ActionEvent.ACTION_PERFORMED, "edit"));				
		}				
	}

	public void mouseExited(MouseEvent e)
	{
		if (e.getSource()!=this)
		{	
			regionLabel.setVisible(false);
		}
	}	
	
	public void mouseClicked(MouseEvent e)
	{
	}		
				
	public void mousePressed(MouseEvent e)
	{
	}		
	
	public void addActionListener(ActionListener listener)
	{
		actionListener = AWTEventMulticaster.add(actionListener, listener);
	}
		
	public void removeActionListener(ActionListener listener)
	{
		actionListener = AWTEventMulticaster.remove(actionListener, listener);
	}				
}
    

    
    
     
     
     
     






