package logica.gui;


import java.util.*;
import java.awt.*;
import java.awt.event.*;

public class NorthNSouth
extends MapVisualization
{

	private RegionBlob[] region = new RegionBlob[buffersize];

	
	//vars in use for mini3D
	double projPlaneD = 600.0F;
	double scalingRatio = 0.9F;
	double yRotation = 1.03F;
	double xRotation = -0.15F;
	double innerDes = 1.1F;
	double outerDes = 1.25F;
	Point3D transRel = new Point3D(0.0F, 0.0F, projPlaneD);
	Point2D screenCntr = new Point2D(515.0F, 155.0F);
	
	//graphical objects
	Point3D[] medianPlane = new Point3D[4];
	Point3D[] northHalfPl = new Point3D[4];
	Point3D[] southHalfPl = new Point3D[4];
	Point3D[] edges = new Point3D[4];
	
	Point2D[] medianPlaneP = new Point2D[4];
	Point2D[] northHalfPlP = new Point2D[4];
	Point2D[] southHalfPlP = new Point2D[4];
	Point2D[] edgesP = new Point2D[4];
	
	
	public Point2D[] projectPlane (Point3D[] plane) {
		Point2D[] retP2Ds = new Point2D[plane.length];
		
		Point3D rotatYT;
		Point3D rotatXT;
		Point3D translT;
		Point2D projecT;
		Point2D scaledT;
		
		for (int i = 0; i < plane.length; i++) {
			rotatYT = plane[i].rotatedYBy(yRotation);
			rotatXT = rotatYT.rotatedXBy(xRotation);
			translT = rotatXT.translatedBy(transRel);
			projecT = translT.projected();
			scaledT = projecT.scaledBy(scalingRatio);
			
			retP2Ds[i] = scaledT.translatedBy(screenCntr);
		}
		
		return retP2Ds;
	}
	
	public void setUpPlanes(double ascent) {
		medianPlane[0] = new Point3D(-300.0F, ascent, -200.0F);
		medianPlane[1] = new Point3D(-300.0F, -ascent, 200.0F);
		medianPlane[2] = new Point3D(300.0F, -ascent, 200.0F);
		medianPlane[3] = new Point3D(300.0F, ascent, -200.0F);
		
		northHalfPl[0] = new Point3D(-300.0F, 0.0F, 200.0F);
		northHalfPl[1] = new Point3D(-300.0F, 0.0F, 0.0F);
		northHalfPl[2] = new Point3D(300.0F, 0.0F, 0.0F);
		northHalfPl[3] = new Point3D(300.0F, 0.0F, 200.0F);
		
		southHalfPl[0] = new Point3D(-300.0F, 0.0F, -200.0F);
		southHalfPl[1] = new Point3D(-300.0F, 0.0F, 0.0F);
		southHalfPl[2] = new Point3D(300.0F, 0.0F, 0.0F);
		southHalfPl[3] = new Point3D(300.0F, 0.0F, -200.0F);
		
		edges[0] = new Point3D(0.0F, -ascent, 200.0F);
		edges[1] = new Point3D(300.0F, 0.0F, 0.0F);
		edges[2] = new Point3D(0.0F, ascent, -200.0F);
		edges[3] = new Point3D(-300.0F, 0.0F, 0.0F);
		
		medianPlaneP = projectPlane(medianPlane);
		southHalfPlP = projectPlane(southHalfPl);
		northHalfPlP = projectPlane(northHalfPl);
		edgesP = projectPlane(edges);
	}

    public NorthNSouth(int width, int height)
    {
		super(width,height);

		setUpPlanes(60.0F);
	}


	public void paint(Graphics g)
	{	
		g.setColor(Color.black);
		//g.fillRect(0,0,w,h);
		Color planeGray = new Color(170,170,170);
		
		double[][] ply, pNH, pMP, pSH;
		Color blend;
		
		pNH = point2Arr(northHalfPlP);
		drawPoly(g, pNH, Style.DarkGrey, Style.MiddleGrey);
		
		pMP = point2Arr(medianPlaneP);
		drawPoly(g, pMP, planeGray, Style.MiddleGrey);
		
		ply = polyCutsPoly(pNH, pMP);
		blend = blendColors(Style.DarkGrey, planeGray, 0.8F);
		if (ply != null) drawPoly(g, ply, blend, Style.MiddleGrey);
		
		pSH = point2Arr(southHalfPlP);
		drawPoly(g, pSH, Style.DarkGrey, Style.MiddleGrey);
		
		ply = polyCutsPoly(pSH, pMP);
		blend = blendColors(planeGray, Style.DarkGrey, 0.8F);
		if (ply != null) drawPoly(g, ply, blend, Style.MiddleGrey);

		g.setColor(Style.MiddleGrey);
		g.setFont( new Font("SansSerif",Font.PLAIN,10));
		FontMetrics fm = getFontMetrics(Style.defaultFont);
		int fontHeight = fm.getHeight();
		
		double xDir, yDir;
		xDir = edgesP[0].x - edgesP[2].x;
		yDir = edgesP[0].y - edgesP[2].y;
		g.drawLine((int)(edgesP[2].x + xDir * innerDes),
		           (int)(edgesP[2].y + yDir * innerDes),
				   (int)(edgesP[0].x - xDir * innerDes),
				   (int)(edgesP[0].y - yDir * innerDes));
		
		g.drawString("NORTH",
		             (int)(edgesP[2].x + xDir * outerDes) - (fm.stringWidth("NORTH") / 2),
		             (int)(edgesP[2].y + yDir * outerDes) + (fontHeight / 2));
		g.drawString("SOUTH",
		             (int)(edgesP[0].x - xDir * outerDes) - (fm.stringWidth("SOUTH") / 2),
				     (int)(edgesP[0].y - yDir * outerDes) + (fontHeight / 2));
		
		xDir = edgesP[1].x - edgesP[3].x;
		yDir = edgesP[1].y - edgesP[3].y;
		g.drawLine((int)(edgesP[3].x + xDir * innerDes),
		           (int)(edgesP[3].y + yDir * innerDes),
				   (int)(edgesP[1].x - xDir * innerDes),
				   (int)(edgesP[1].y - yDir * innerDes));
		
		g.drawString("EAST",
		             (int)(edgesP[3].x + xDir * outerDes) - (fm.stringWidth("EAST") / 2),
		             (int)(edgesP[3].y + yDir * outerDes) + (fontHeight / 2));
		g.drawString("WEST",
		             (int)(edgesP[1].x - xDir * outerDes) - (fm.stringWidth("WEST") / 2),
				     (int)(edgesP[1].y - yDir * outerDes) + (fontHeight / 2));
		
		super.paint(g);
	}


	public void addRegion(String name, int id, float x, float y, float value)
	{
		region[nrOfRegions] = new RegionBlob(name,
											id,
											w/2 + (x/360 * w),
											h/2 + (y/180 * h),
											deNaN(value));
		region[nrOfRegions].addMouseListener(this);
		double[] spatialC = {x, deNaN(value), -y};
		region[nrOfRegions].setSpatialCoords(spatialC);
		nrOfRegions++;
		if (nrOfRegions >= buffersize)
		{
			buffersize += 10;
			RegionBlob[] temp = new RegionBlob[buffersize];
			System.arraycopy(region,0,temp,0,region.length);
			region = temp;
		}
	}



    void arrange()
    {
		removeAll();

		setLayout(null);
		setSize(w,h);

		add(regionLabel);

		sortByValue(region);
		scaleToFit(region, 0.0F, 1.0F);

		for(int i = 0; i < nrOfRegions; ++i)
		{
			add(region[i]);
		}
		repaint();
	}

	public void reArrange()
	{
		arrange();
	}


	public void setNewData(float[] values)
	{
		sortById(region);
		for (int i = 0; i < nrOfRegions; ++i)
		{
			if (i < values.length) {
				region[i].setValue(values[i]);
				region[i].setAbsValue(values[i]);
			}
			else {
				region[i].setValue(0);								// ist der array zu kurz wert auf null setzen
				region[i].setAbsValue(0);							// ueberschuessige elemnte zu langer array werden ignoriert
			}
		}
		arrange();
	}


	public void activate(int id)
	{
		for (int i = 0; i < nrOfRegions; i++)
		{
			if (region[i].getActive()) region[i].setActive(false);
			if (region[i].getId() == id) region[i].setActive(true);
		}
	}
	
	
	//dummy for satisfaction of inheritance...; sepp 020330
	void scaleToFit(MapElement[] object, float min, float max) {}
	

    void scaleToFit(RegionBlob[] object, float min, float max)
    {
		Point3D[] projPosT = new Point3D[2];	//proj1, proj2;
		Point3D blbPosT;
		Point3D rotatYT, rotatXT, translT;
		Point2D[] projPosP = new Point2D[2];	//proj1, proj2;
		Point2D projecT, scaledT, positnT;
			
		double[] spatialC;
		double[] positn = new double[3];
		int x, y;
		int proj1Dist, proj2Dist;				//distances of projections
		
        float v_max = 0.01F;
		float v_min = 1.0E17F;
		float v_span;
		float northVals = 0.0F;
		float southVals = 0.0F;
		int northC = 0;
		int southC = 0;
		double ascent = 0.0F;
		float pseudoAeq = 10.0F;		//most regions lie north of the real aequator...
		
		for (int i=0; i < nrOfRegions; i++) {
			object[i].setValue(deNaN(object[i].getValue()));
			
			v_max = Math.max(object[i].getValue(),v_max);
			v_min = Math.min(object[i].getValue(),v_min);
		}
		
		v_min = Math.max(0.01F, v_min);
		v_span = v_max - v_min;

        for (int i=0; i < nrOfRegions; i++) {
			//pow for diminuation of differences
			object[i].setValue(Math.max(object[i].getValue(), 0.01F));
			object[i].setValue((float)Math.pow((((object[i].getValue() - v_min)/ v_span) * max), 0.3));
			//if (object[i].getValue() < min) object[i].setValue(min);

			spatialC = object[i].getSpatialCoords();
			
			//setting global position
			positn[0] = (spatialC[0] / 180) * 300;
			positn[2] = (spatialC[2] / 90) * 200;
			//setting value
			spatialC[1] = object[i].getValue();
			positn[1] = (spatialC[1] - 0.5F) * -160;
			
			//computation of ascent
			if (spatialC[2] > pseudoAeq) {
				southVals += positn[1];
				southC++;
			}
			else {
				northVals += positn[1];
				northC++;
			}
			
			//project spatial position
			blbPosT = new Point3D(positn[0], positn[1], positn[2]);
			rotatYT = blbPosT.rotatedYBy(yRotation);
			rotatXT = rotatYT.rotatedXBy(xRotation);
			translT = rotatXT.translatedBy(transRel);
			projecT = translT.projected();
			scaledT = projecT.scaledBy(scalingRatio);
			positnT = scaledT.translatedBy(screenCntr);
			
			x = (int)positnT.x;
			y = (int)positnT.y;		
			
			object[i].rePosition(x,y);
        }
		
		northVals /= northC;
		southVals /= southC;
		ascent = (northVals - southVals) / 160.0F;
		
		//amplifying ascent
		ascent = deNaN((float)((ascent / Math.abs(ascent)) * Math.pow(Math.abs(ascent), 0.6F)));
		
		//project lines through planes
		for (int i=0; i < nrOfRegions; i++) {
			spatialC = object[i].getSpatialCoords();
			
			positn[0] = (spatialC[0] / 180) * 300;
			positn[2] = (spatialC[2] / 90) * 200;
			positn[1] = ascent * 160 * -positn[2] / 200;
			
			projPosT[0] = new Point3D(positn[0], positn[1], positn[2]);
			projPosT[1] = new Point3D(positn[0], 0.0F,      positn[2]);
			
			projPosP = projectPlane(projPosT);
			
			proj1Dist = (int)(projPosP[0].y) - object[i].getY();
			proj2Dist = (int)(projPosP[1].y) - object[i].getY();
			
			object[i].setLines(proj1Dist, proj2Dist);
		}
		
		setUpPlanes(ascent * 160.0F);
    }
	
	
	//utility-Classes
	public class Point3D {
		double x;
		double y;
		double z;
		
		public Point3D (double xArg, double yArg, double zArg) {
			x = xArg;
			y = yArg;
			z = zArg;
		}
		
		public Point3D () {
			x = y = z = 0.0F;
		}
		
		public Point3D translatedBy(Point3D transl) {
			Point3D retP3D = new Point3D(x, y, z);
			
			retP3D.x += transl.x;
			retP3D.y += transl.y;
			retP3D.z += transl.z;
			
			return retP3D;
		}
		
		public Point3D rotatedXBy(double rotation) {
			Point3D retP3D = new Point3D();

			retP3D.x = x;
			retP3D.y = y * Math.cos(rotation) + z * Math.sin(rotation);
			retP3D.z = y * (-1.0F) * Math.sin(rotation) + z * Math.cos(rotation);

			return retP3D;
		}
		
		public Point3D rotatedYBy(double rotation) {
			Point3D retP3D = new Point3D();

			retP3D.x = x * Math.cos(rotation) - z * Math.sin(rotation);
			retP3D.y = y;
			retP3D.z = x * Math.sin(rotation) + z * Math.cos(rotation);

			return retP3D;
		}
		
		public Point2D projected() {
			Point2D retP2D = new Point2D();
			
			retP2D.x = x / (z / projPlaneD);
			retP2D.y = y / (z / projPlaneD);
			
			return retP2D;
		}
	}
	
	//code for polygon intersection
	public double leftOrRight (double[][] line, double[] testPoint) {
		double a = line[0][1] - line[1][1];
		double b = line[1][0] - line[0][0];
		double c = ((-a * line[0][0]) + (-b * line[0][1]));

		return ((a * testPoint[0]) + (b * testPoint[1]) + c);
	}


	public double[] intersect (double[][] lineA, double lineB[][]) {
		double x1a = lineA[0][0];
		double x2a = lineA[1][0];
		double y1a = lineA[0][1];
		double y2a = lineA[1][1];
		double x1b = lineB[0][0];
		double x2b = lineB[1][0];
		double y1b = lineB[0][1];
		double y2b = lineB[1][1];

		double u =   ((x2b - x1b) * (y1a - y1b) - (y2b - y1b) * (x1a - x1b))
		           / ((y2b - y1b) * (x2a - x1a) - (x2b - x1b) * (y2a - y1a));

		double[] result = new double[2];

		result[0] = x1a + u * (x2a - x1a);
		result[1] = y1a + u * (y2a - y1a);

		return result;
	}


	public double[][] lineCutsPoly (double[][] poly, double[][] line) {
		int pLen = poly.length;
		int vCount = 0;
		double[][] segment = new double[2][2];
		double[][] result = new double[pLen + 1][2];
		double lor1, lor2;

		for (int i = 0; i < pLen; i++) {
			segment[0] = poly[i];
			segment[1] = poly[(i + 1) % pLen];

			lor1 = leftOrRight(line, segment[0]);
			lor2 = leftOrRight(line, segment[1]);

			if (lor1 >= 0.0F) {
				result[vCount  ][0] = poly[i][0];
				result[vCount++][1] = poly[i][1];
			}

			if (lor1 * lor2 < 0.0F) {
				result[vCount++] = intersect(segment, line);
			}
		}

		double[][] cutResult = new double[vCount][2];
		for (int i = 0; i < vCount; i++) cutResult[i] = result[i];

		if (vCount > 2) return cutResult;
		else return null;
	}


	public double[][] polyCutsPoly (double[][] polyA, double[][] polyB) {
		int pLen = polyA.length;
		double[][] segment = new double[2][2];

		for (int i = 0; i < pLen; i++) {
//			if (polyB.length > 2) {
			if (polyB != null) {
				segment[0] = polyA[i];
				segment[1] = polyA[(i + 1) % pLen];

				polyB = lineCutsPoly(polyB, segment);
			}
		}

		return polyB;
	}


	public int[][] xNyVals(double[][] poly) {
		int[][] retVals = new int[2][poly.length];
		for (int i = 0; i < poly.length; i++) {
			retVals[0][i] = (int)Math.round(poly[i][0]);
			retVals[1][i] = (int)Math.round(poly[i][1]);
		}
		return retVals;
	}
	
	
	public void drawPoly(Graphics graph, double[][] poly, Color fillCol, Color borderCol) {
		graph.setColor(fillCol);
		int length = poly.length;
		int[][] vecs = xNyVals(poly);
		graph.fillPolygon(vecs[0], vecs[1], length);
		graph.setColor(borderCol);
		if (borderCol != null) graph.drawPolygon(vecs[0], vecs[1], length);
	}

	//stupid workaround...; sepp 020328
	public double[][] point2Arr(Point2D[] pointPoly) {
		double[][] retArr = new double[pointPoly.length][2];
		for (int i = 0; i < pointPoly.length; i++) {
			retArr[i][0] = pointPoly[i].x;
			retArr[i][1] = pointPoly[i].y;
		}
		return sortClockWise(retArr);
	}
	
	//another stupid workaround...; sepp 020328
	public double[][] sortClockWise(double[][] poly) {
		double[][] testline = new double[2][2];
		testline[0] = poly[0];
		testline[1] = poly[1];
		if (leftOrRight(testline, poly[2]) < 0.0F) {
			int pLen = poly.length;
			double[][] cwPoly = new double[pLen][2];
			for (int i = 0; i < pLen; i++) {
				cwPoly[i][0] = poly[pLen - i - 1][0];
				cwPoly[i][1] = poly[pLen - i - 1][1];
			}
			return cwPoly;
		}
		else return poly;
	}


	//THE stupid workaround...; sepp 020403
	public float deNaN(float value) {
		if (Float.isNaN(value)) return 0.0F;
		else return value;
	}
	
	
	//code for faking transparency
	public Color blendColors(Color a, Color b, double blend) {
		int iBlend, iBlendInv, red, green, blue;
		iBlend = (int)(255 * blend);
		iBlendInv = 255 - iBlend;
		red   = ((a.getRed()   * iBlend + b.getRed()   * iBlendInv) >> 8);
		green = ((a.getGreen() * iBlend + b.getGreen() * iBlendInv) >> 8);
		blue  = ((a.getBlue()  * iBlend + b.getBlue()  * iBlendInv) >> 8);
		return new Color(red, green, blue);
	}
	
	
	//the following has nothing to do with the one supplied by java itself
	public class Point2D {
		double x;
		double y;
		
		public Point2D (double xArg, double yArg) {
			x = xArg;
			y = yArg;
		}
		
		public Point2D () {
			x = y = 0.0F;
		}
		
		public Point2D translatedBy(Point2D transl) {
			Point2D retP2D = new Point2D(x, y);
			
			retP2D.x += transl.x;
			retP2D.y += transl.y;
			
			return retP2D;
		}
		
		public Point2D scaledBy(double ratio) {
			Point2D retP2D = new Point2D(x, y);
			
			retP2D.x *= ratio;
			retP2D.y *= ratio;
			
			return retP2D;
		}
	}
}










