package logica.gui;

import logica.utility.*;
import logica.sim.io.*;
import logica.sim.*;

import java.util.*;
import java.awt.*;
import java.awt.event.*;

public class RegionControlPanel
extends Container
implements ActionListener, MouseListener, MouseMotionListener
{
	private int w;
	private int h;
	
	private int headerH;
	
	private String name = "name";	
	private int regionNr = 0;

	private SimOutput simOut;
	private UIHandler uiHandler;	
	private Simulation sim;
		
	private FlyingLabel label;
	private LSlider[] slider;
	private LMultiSlider distSlider;
	private CloseButton closeButton;
	private MinimizeButton minimizeButton;
		
	private NullComponent blocker;
	private NullComponent header;
	
	private ActionListener actionListener;

	private boolean collapsed = false;
	private int margin = 2;
	private int actYear;	
	FontMetrics fm;
        
    public RegionControlPanel(Simulation sim)
    {		
		super();
		
		this.sim = sim;
		this.simOut = sim.sOut;
		this.uiHandler = sim.sUI;
		
		this.w = 370;
		this.h = 120;
							
		label = new FlyingLabel("");
		
		blocker = new NullComponent(w,h);
		
		closeButton = new CloseButton();
		closeButton.addActionListener(this);
		
		minimizeButton = new MinimizeButton();
		minimizeButton.addActionListener(this);		
		
		slider = new LSlider[3];
		
		distSlider = new LMultiSlider();
		distSlider.addSlider("industry", 0.25F);
		distSlider.addSlider("agriculture", 0.25F);
		distSlider.addSlider("hightech", 0.25F);
		distSlider.addSlider("other", 0.25F);		
		distSlider.addActionListener(this);
		
		setFont(Style.defaultFont);
		fm = getFontMetrics(Style.defaultFont);	

		headerH = Style.defaultFontHeight + 2*margin + 1;
		
		header = new NullComponent(w,headerH);
		header.addMouseMotionListener(this);
		header.addMouseListener(this);		
		
		arrange();
	}
	
		
		
    private void arrange()
    {
		removeAll();
		
		setLayout(null);	
		setSize(w,h);
		
		add(label);
		label.setVisible(false);
			
		closeButton.setLocation(w - (closeButton.getWidth() + 2),(headerH - closeButton.getHeight())/2);
		add(closeButton);
		minimizeButton.setLocation(w - 2*(closeButton.getWidth() +2),(headerH - minimizeButton.getHeight())/2);
		add(minimizeButton);		
		
		distSlider.setLocation(10,55);
		add(distSlider);
		
		slider[0] = new LSlider("foodAid");
		slider[0].setLocation(220,55);
		slider[0].addActionListener(this);
		add(slider[0]);
		
		slider[1] = new LSlider("utilAid");
		slider[1].setLocation(220,95);
		slider[1].addActionListener(this);
		add(slider[1]);		

		header.setLocation(0,0);
		add(header);
		
		blocker.setLocation(0,0);
		add(blocker);
	}
	
	
	public void paint(Graphics g)
	{	
		g.setFont(Style.defaultFont);

		//g.setColor(Style.DarkGrey);
		g.setColor(Color.black);
		g.fillRect(0,0,w,h);
		
		g.setColor(Style.bgGreen1);
		g.fillRect(0, 0, w, headerH);		
		
		g.setColor(Style.defaultBorder);
		g.drawRect(0, 0, w-1, headerH);		
		
		g.drawRect(0, headerH + 2 , w-1, h-headerH-3);	
				
		g.setColor(Color.white);
		g.drawString(name, 5, Style.defaultFontHeight);	
		g.setColor(Style.textLight);
		g.drawString("| CONTROL PANEL",fm.stringWidth(name)+10,Style.defaultFontHeight);			
			
		g.setColor(Style.LightGrey);
		g.drawString("target investment distribution",10,45);	
		
		g.drawString("food aid (% of surplus)",220,45);				
		g.drawString("financial aid (% of surplus)",220,85);						
		
		super.paint(g);
	}	
		
	
	public void loadData(int nr, int yr)
	{		
		regionNr = nr;		
		name = simOut.getName(regionNr);
		uiHandler.setPresentYear(yr);			
		
		float[] ui_val = {
			uiHandler.getUserSliderVal("targetS_industBase", regionNr),
			uiHandler.getUserSliderVal("targetS_agriEquip", regionNr),
			uiHandler.getUserSliderVal("targetS_advTech", regionNr),
			uiHandler.getUserSliderVal("targetS_other", regionNr) };
		
		float[] sh_val = {
			uiHandler.getSliderVal("targetS_industBase", regionNr),
			uiHandler.getSliderVal("targetS_agriEquip", regionNr),
			uiHandler.getSliderVal("targetS_advTech", regionNr),
			uiHandler.getSliderVal("targetS_other", regionNr) };		
		
		
		distSlider.setValues(ui_val);
		distSlider.setSecondValues(sh_val);
		
		slider[0].setValue(uiHandler.getUserSliderVal("foodAid", regionNr));
		slider[0].setSecondValue(uiHandler.getSliderVal("foodAid", regionNr));

		slider[1].setValue(uiHandler.getUserSliderVal("utilAid", regionNr));
		slider[1].setSecondValue(uiHandler.getSliderVal("utilAid", regionNr));					
	}		
	
	public void setActYear(int yr)
	{		
		//actYear = yr;
		//uiHandler.setPresentYear(actYear);			
	}	
	
	public void toggleCollapse()
	{
		collapsed = !collapsed;
		if (collapsed) this.setSize(w,headerH+1);
		else 
		{
			this.setSize(w,h);			
			Point Loc = this.getLocation();
			if ( Loc.y > getParent().getSize().height - h - 5)
			{
				this.setLocation(Loc.x, getParent().getSize().height - h - 5);
			}
		}
		repaint();		
	}
	
	public int getWidth()
	{
		return w;
	}
	
	public int getHeight()
	{
		return h;
	}		
	

	
	public void actionPerformed(ActionEvent e)
	{
		String cmd = e.getActionCommand();
		if(cmd.equals("close"))		
		{
			this.setVisible(false);
			repaint();
		}
		if(cmd.equals("minimize"))		
		{
			this.toggleCollapse();
		}		
		else if(cmd.equals("value_change"))
		{
			LSlider source = (LSlider)e.getSource() ;
			label.setText(((int)(source.getValue()*100)) + "%");
			
			Point newPos = new Point(source.getLocation());
			newPos.translate(source.getPos() - label.getWidth() / 2, - label.getHeight());			
			label.setLocation(newPos);
			label.setVisible(true);
			repaint();
		}			
		else if(cmd.equals("dist_value_change"))
		{
			LMultiSlider sourceDist = (LMultiSlider) e.getSource();
			LSlider source = sourceDist.getActual();
			
			sourceDist.distributeValueChange();

			label.setText(((int)(source.getValue()*100)) + "%");			
			Point newPos = new Point(source.getLocation());
			Point DistPos = new Point(sourceDist.getLocation());
			newPos.translate(DistPos.x + source.getPos() - label.getWidth() / 2, DistPos.y - label.getHeight());						
			label.setLocation(newPos);
			label.setVisible(true);					
			repaint();
		}			
		else if(cmd.equals("change_done"))
		{
			LSlider source = (LSlider) e.getSource();
			label.setVisible(false);
									
			uiHandler.submitValue(source.getName(), regionNr, source.getValue());
			
			if(actionListener != null) 
			{
				actionListener.actionPerformed(new ActionEvent(this, ActionEvent.ACTION_PERFORMED, "user_change_done"));				
			}										
		}
		else if(cmd.equals("dist_change_done"))
		{
			LMultiSlider sourceDist = (LMultiSlider) e.getSource();
			label.setVisible(false);

			uiHandler.submitDistValue("targetS", regionNr, sourceDist.getValues());

			if(actionListener != null) 
			{
				actionListener.actionPerformed(new ActionEvent(this, ActionEvent.ACTION_PERFORMED, "user_change_done"));				
			}							
			repaint();
		}			
	}
	
	
	// Mouse Listening
	
	
	private boolean dragging = false;
	private int oldX,oldY;
			
	public void mouseDragged(MouseEvent e) 
	{
		if (dragging)
		{
			Point oldLoc = getLocation();
			int xPos = Math.max(0, oldLoc.x + e.getX()-oldX);
			xPos = Math.min(xPos, getParent().getSize().width - w);
			int yPos = Math.max(0, oldLoc.y + e.getY()-oldY);
			yPos = Math.min(yPos, getParent().getSize().height - headerH - 1);			
			this.setLocation(xPos,yPos);
			repaint();
		}			
	}
	public void mousePressed(MouseEvent e) 
	{
		if (e.getY() < headerH)
		{
			oldX = e.getX();
			oldY = e.getY();
			dragging = true;
		}
	}
		
	public void mouseReleased(MouseEvent e) 
	{
		dragging = false;
	}
	
	public void mouseEntered(MouseEvent e) 	{}
		
	public void mouseExited(MouseEvent e) {}
		
	public void mouseMoved(MouseEvent e) {}

	public void mouseClicked(MouseEvent e) 
	{
		if (e.getClickCount() >1) this.toggleCollapse();
	}	
	
	
	
	public void addActionListener(ActionListener listener)
	{
		actionListener = AWTEventMulticaster.add(actionListener, listener);
	}
		
	public void removeActionListener(ActionListener listener)
	{
		actionListener = AWTEventMulticaster.remove(actionListener, listener);
	}		
}
