package logica.gui;

import java.awt.*;
import java.awt.event.*;


public class ResetButton 
extends Component
implements MouseListener
{
	private String name;
	private String actionCommand;

		
	private int w;
	private int h;

	private FontMetrics fm;
	private int topMargin = 2;
	private int leftMargin = 10;
	
	private boolean hasFocus;
	private boolean isActive;	
	private boolean isValid;	
	
	private ActionListener actionListener;

	public ResetButton()
	{
		this("RESET");
	}

	
	public ResetButton(String name)
	{
		super();
        
		this.name = name;
		hasFocus = false;
		isActive = false;
		isValid = true;
		
		setFont(Style.defaultFont);
		fm = getFontMetrics(Style.defaultFont);
		
		h = Style.defaultFontHeight + (2 * topMargin);
		w = fm.stringWidth(name) + (2 * leftMargin);		
		setSize(w,h);		
		
		setActionCommand(name);
		
		enableEvents(AWTEvent.MOUSE_EVENT_MASK);
		
		addMouseListener(this);
	}
        	
	
	public String getName()
	{
		return name;
	}	
	
	public void setWidth(int width)
	{
		w = width;
		setSize(w,h);
	}				

	
	public void setActionCommand(String cmd)
	{
		actionCommand = cmd;
	}
	
	public String getActionCommand()
	{
		return actionCommand;
	}
	
	public void setActive(boolean stat)
	{
		isActive = stat;
		repaint();
	}
	
	
	public boolean getActive()
	{
		return isActive;
	}
			
	public void setValid(boolean stat)
	{
		isValid = stat;
		repaint();
	}	

	
	public void setMargin(int top, int left)
	{
		this.topMargin = top;
		this.leftMargin = left;		
	}	
	
	public Dimension getPreferredSize()
	{
		return new Dimension(w,h);
	}
	

	public Dimension getMinimumSize()
	{
		return new Dimension(w,h);
	}
	
	public int getWidth()
	{
		return w;
	}
	
	public int getHeight()
	{
		return h;
	}
	
	
	public boolean isFocusTraversable()
	{
		return true;
	}	

	
	public void paint(Graphics g)
	{
		g.setFont(Style.defaultFont);
		if(!isValid)
		{
			g.setColor(Style.MiddleGrey);
			g.drawString(name,leftMargin, Style.defaultFontHeight);
			g.drawRect(0,0,w-1,h-1);			
		}			
		else if (isActive)
		{
			g.setColor(Style.textLight);
			g.drawString(name,leftMargin, Style.defaultFontHeight);
			g.setColor(Style.textLight);			
			g.drawRect(0,0,w-1,h-1);				
		}
		else if(hasFocus)
		{
			g.setColor(Style.menuText);
			g.drawRect(0,0,w-1,h-1);					
			g.setColor(Style.menuText);
			g.drawString(name,leftMargin, Style.defaultFontHeight);			
		}	
		else 
		{
			g.setColor(Style.defaultBorder);			
			g.drawRect(0,0,w-1,h-1);			
			g.setColor(Style.textLight);		
			g.drawString(name,leftMargin, Style.defaultFontHeight);				
		}		  
	}

	public void addActionListener(ActionListener listener)
	{
		actionListener = AWTEventMulticaster.add(actionListener, listener);
		enableEvents(AWTEvent.MOUSE_EVENT_MASK);
	}
		
	public void removeActionListener(ActionListener listener)
	{
		actionListener = AWTEventMulticaster.remove(actionListener, listener);
	}		
	

		
	public void mouseClicked(java.awt.event.MouseEvent e) 
	{
	}	

	public void mouseEntered(java.awt.event.MouseEvent e) 
	{
		if (isValid)
		{
			requestFocus();	
			hasFocus = true;
			repaint();
		}
	}
	
	public void mouseExited(java.awt.event.MouseEvent e) 
	{
		if (isValid)
		{	
			hasFocus = false;
			repaint();
		}
	}
	
	public void mousePressed(java.awt.event.MouseEvent e) 
	{
	}
	
	public void mouseReleased(java.awt.event.MouseEvent e)
	{
		if (isValid)
		{
			if(actionListener != null) 
			{
				actionListener.actionPerformed(new ActionEvent(this, ActionEvent.ACTION_PERFORMED, name));				
			}			
			repaint();			
		}
	}
	

}
	 


