package logica.gui;

import java.awt.*;
import java.awt.event.*;


public class TimeSlider
extends Component
implements MouseMotionListener, MouseListener
{		
	private int sliderH = 7;	
	private int pos;
	
	private int w;
	private int h;

	private int year;
	private int actYear;	
	private int maxYear;
	
	private int segW = 4;
	private int leftPos = 100;
	
	private boolean hasFocus;
	private boolean isActive;	
	
	private ActionListener actionListener;

	
	public TimeSlider()
	{
		this(0,100);
	}
	
	public TimeSlider(int year)
	{
		this(year, 100);
	}
	
	public TimeSlider(int actYear,  int maxYear)
	{
		super();
        
		this.year = actYear;
		this.actYear = actYear;
		this.maxYear = maxYear;
		
		w = 800;
		h = Style.defaultFontHeight +  12;
		setSize(w,h);
		
		this.pos = leftPos + (this.year * segW);
		
		hasFocus = false;
		isActive = false;				
				
		addMouseMotionListener(this);		
		addMouseListener(this);
	}
        
	
	public void setYear(int yr)
	{
		year = yr;
	}	
	
	public void setActYear(int yr)
	{
		actYear = yr;
	}		

	public void setMaxYear(int yr)
	{
		maxYear = yr;
	}			
	
	public int getYear()
	{
		return year;
	}	

	public int getPos()
	{
		return pos;
	}		
	
	public int getWidth()
	{
		return w;
	}
	
	public int getHeight()
	{
		return h;
	}
	
	
	public boolean isFocusTraversable()
	{
		return true;
	}	

	
	public void paint(Graphics g)
	{		
		g.setFont(Style.defaultFont);		
		FontMetrics fm = getFontMetrics(Style.defaultFont);
				
		g.setColor(Style.textLight);		
		g.drawLine(0,Style.defaultFontHeight - 2, 3, Style.defaultFontHeight - 5);	
		g.drawLine(0,Style.defaultFontHeight - 8, 3, Style.defaultFontHeight - 5);
		

		g.setColor(Style.textDark);		
		g.drawString("TIMELINE ", 20, Style.defaultFontHeight);

		for (int i = 0; i<=maxYear; i++)
		{
			if (i == actYear) g.setColor(Color.white);
			if (i == actYear+1) g.setColor(Style.textLight);
			g.fillRect(leftPos + (segW* i) , Style.defaultFontHeight - 5, 3, 3);
		}

		
		g.setColor(Color.white);
		int[] arx = {pos-3, pos, pos + 5};
		int[] ary = {Style.defaultFontHeight - 11, Style.defaultFontHeight - 7, Style.defaultFontHeight - 11};		
		g.fillPolygon(arx,ary,arx.length);		
		
		int tempPos = (leftPos + actYear*segW);
		int[] arx2 = {tempPos - 3, tempPos, tempPos+ 5};
		int[] ary2 = {Style.defaultFontHeight + 3, Style.defaultFontHeight -1, Style.defaultFontHeight + 3};		
		g.fillPolygon(arx2,ary2,arx2.length);			
							
	}

	public void addActionListener(ActionListener listener)
	{
		actionListener = AWTEventMulticaster.add(actionListener, listener);
	}
		
	public void removeActionListener(ActionListener listener)
	{
		actionListener = AWTEventMulticaster.remove(actionListener, listener);
	}		
	

	public void mouseDragged(MouseEvent e)
	{
		isActive = true;
		pos = e.getX();
		pos = Math.max(leftPos, pos);
		pos = Math.min(leftPos + maxYear*segW, pos);
		year = (int) ((double) (pos - leftPos)/ (double) segW);
		if(actionListener != null) 
		{
				actionListener.actionPerformed(new ActionEvent(this, ActionEvent.ACTION_PERFORMED, "time_change"));				
		}				
		repaint();
	}		

	public void mouseMoved(MouseEvent e) {}		
	
	public void mouseClicked(MouseEvent e) {}
	
	public void mouseEntered(MouseEvent e) 
	{
		hasFocus = true;
		requestFocus();
	}
	
	public void mouseExited(MouseEvent e) 
	{
		hasFocus = false;
		isActive = false;
		//repaint();
	}
	
	public void mousePressed(MouseEvent e) 
	{
		isActive = true;
		pos = e.getX();
		pos = Math.max(leftPos, pos);
		pos = Math.min(leftPos + maxYear*segW, pos);
		year = (int) ((double) (pos - leftPos)/ (double) segW);
		if(actionListener != null) 
		{
				actionListener.actionPerformed(new ActionEvent(this, ActionEvent.ACTION_PERFORMED, "time_change"));				
		}						
		repaint();	
	}
	
	public void mouseReleased(MouseEvent e) 
	{
		isActive = false;
		if(actionListener != null) 
		{
				actionListener.actionPerformed(new ActionEvent(this, ActionEvent.ACTION_PERFORMED, "change_done"));				
		}			
		repaint();		
	}
	
}
	
