package logica.gui;

import java.awt.*;
import java.awt.event.*;


public class TxTButton 
extends Component
implements MouseListener
{
	private String name;
	private int id;	
	private String actionCommand;

		
	private int w;
	private int h;

	private FontMetrics fm;
	private int topMargin = 2;
	private int leftMargin = 10;
	
	private boolean hasFocus;
	private boolean isActive;	
	
	private ActionListener actionListener;

	
	public TxTButton(String name)
	{
		this(name,-1);
	}
	
	public TxTButton(String name,int id)
	{
		super();
        
		this.name = name;
		this.id = id;
		hasFocus = false;
		isActive = false;
		
		setFont(Style.defaultFont);
		fm = getFontMetrics(Style.defaultFont);
		
		h = Style.defaultFontHeight + (2 * topMargin);
		w = fm.stringWidth(name) + (2 * leftMargin);		
		setSize(w,h);		
		
		setActionCommand(name);
		
		enableEvents(AWTEvent.MOUSE_EVENT_MASK);
		
		addMouseListener(this);
	}
        

	public int getId()
	{
		return id;
	}	
	
	
	public String getName()
	{
		return name;
	}	
	
	public void setWidth(int width)
	{
		w = width;
		setSize(w,h);
	}		
	
	
	public double getValue()
	{
		return 0;
	}			

	
	public void setActionCommand(String cmd)
	{
		actionCommand = cmd;
	}
	
	public String getActionCommand()
	{
		return actionCommand;
	}
	
	public void setActive(boolean stat)
	{
		isActive = stat;
		repaint();
	}
	
	
	public boolean getActive()
	{
		return isActive;
	}
			
	
	public void setValue(double v)
	{
	}	
	
	
	public void setMargin(int top, int left)
	{
		this.topMargin = top;
		this.leftMargin = left;		
	}	
	
	public Dimension getPreferredSize()
	{
		return new Dimension(w,h);
	}
	

	public Dimension getMinimumSize()
	{
		return new Dimension(w,h);
	}
	
	public int getWidth()
	{
		return w;
	}
	
	public int getHeight()
	{
		return h;
	}
	
	
	public boolean isFocusTraversable()
	{
		return true;
	}	

	
	public void paint(Graphics g)
	{
		g.setFont(Style.defaultFont);
		if (isActive)
		{
			g.setColor(Style.menuActive);
			g.drawString(name,leftMargin, Style.defaultFontHeight);
			g.setColor(Style.textLight);			
			g.drawRect(0,0,w-1,h-1);				
		}
		else if(hasFocus)
		{
			g.setColor(Style.menuActive);
			g.drawString(name,leftMargin, Style.defaultFontHeight);			
		}
		else 
		{
			g.setColor(Style.textLight);		
			g.drawString(name,leftMargin, Style.defaultFontHeight);				
		}		  
	}

	public void addActionListener(ActionListener listener)
	{
		actionListener = AWTEventMulticaster.add(actionListener, listener);
		enableEvents(AWTEvent.MOUSE_EVENT_MASK);
	}
		
	public void removeActionListener(ActionListener listener)
	{
		actionListener = AWTEventMulticaster.remove(actionListener, listener);
	}		
	

		
	public void mouseClicked(java.awt.event.MouseEvent e) 
	{
	}	

	public void mouseEntered(java.awt.event.MouseEvent e) 
	{
			requestFocus();	
			hasFocus = true;
			repaint();		
	}
	
	public void mouseExited(java.awt.event.MouseEvent e) 
	{
			hasFocus = false;
			repaint();		
	}
	
	public void mousePressed(java.awt.event.MouseEvent e) 
	{
	}
	
	public void mouseReleased(java.awt.event.MouseEvent e)
	{
			if(actionListener != null) 
			{
				actionListener.actionPerformed(new ActionEvent(this, ActionEvent.ACTION_PERFORMED, name));				
			}			
			repaint();			
	}
	

}
	 


