/*

---------------------------------------------------------
:/logicaland 0.1
partcipative global simulation
---------------------------------------------------------
Copyright (C) 2002 ./logicaland,
Michael Aschauer, Josef Deinhofer (io@logicaland.net)
---------------------------------------------------------
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.
 
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

For information on usage and redistribution, and for a DISCLAIMER OF ALL
WARRANTIES, see the file, "license.txt," in this distribution.  

----------------------------------------------------------
The model used is RW3 which stands for Regionalized World 3.
RW3 was developed by Fred Kile with the programming assistance 
of Arnold Rabehl and Lori Schaefer.
In 1992 Peter Brecke completed the porting of this model from 
an IBM 370/158 mainframe to the IBM PC class of computers. 
The copyright for RW-3 belongs to Frederick Kile. 
Please acknowledge the model creator in any publication that makes 
use of this model or a derivative of it.

---------------------------------------------------------
*/

package logica.sim;

import logica.utility.*;
import logica.sim.io.*;
import logica.sim.data.*;

//DEBUG
import java.io.*;


public class Simulation {
	public String codebase;

	public Vars v;
	public Year[] data;
	public Utility u;
	public Calc calc;
	public SimInput sInput;
	public SimOutput sOut;
	public UIHandler sUI;		//mash	
	
	private boolean didInit = false;


	public Simulation (String codebaseArg, Utility uArg) {
		u = uArg;
		v = new Vars();

		codebase = codebaseArg;

		data = new Year[999];
		calc = new Calc(data, v, u);
		sOut = new SimOutput(data, v, u);
		sInput = new SimInput(codebase, u);
		
		v.setNumRegs(sInput.getNumOfRegs());
		
		sUI = new UIHandler(data, v); 	//sepp 020331
		
	}
	
	
	public int getNumRegs() {
		return v.numRegs;
	}
	
	
	public int getActYr() {
		return v.calcYr;
	}

	
	public void reCalc(int years) {
		rewind(v.calcYr - years);
		calculate(years);
	}
	
	
	public void rewind(int year) {	
		v.calcYr = year; // Math.min(v.calcYr, (Math.max(year, v.baseYr)));
	}
	
	
	public void calculate(int numOfYrs) {
		if (!didInit)  {
			prepare();
		//	numOfYrs -= 6;
		}
		for (int i = 0; i < numOfYrs; i++) compute(false);
	}
	

	public void prepare() {
		v.calcYr = 0;
		
		while (v.calcYr <= v.prepYrs) data[v.calcYr++] = new Year(v);
		
		v.calcYr--;
		
		sInput.loadRegs(data[v.calcYr].r);
		compute(true);
		sInput.loadRegs(data[v.calcYr].r);
		calc.initRegions();
		calc.worldSetVarsA();
		
		didInit = true;
	}



	public void compute(boolean firstYr) {
		v.calcYr++;
		data[v.calcYr] = new Year(v);
		data[v.calcYr].isCopyOf(data[v.calcYr - 1]);
		
		//DEBUG 020312
//		Runtime.getRuntime().gc();
		
		if (v.calcYr > (v.prepYrs + 1)) sUI.incorporateUI();
		
		if (firstYr) calc.initRegions();			//initialise regions
		if (firstYr) calc.worldSetVarsA();			//set some worldData
		calc.regClassTechTrain();					//classification, tech- & training level
		if (firstYr) calc.regPrevIndustProdFYR();	//create some history
		calc.regFoodGrowthCalcA();					//food
		if (!firstYr) calc.regFoodGrowthCalcB();
		calc.regFood();
		calc.regEmigImmigReset();					//resetting some variables
		if (firstYr) calc.worldFoodSurpSetFYR();	//setting last year value
		if (firstYr) calc.regGNPFYR();				//GNP
		else         calc.regGNP();
		calc.regRankByIncome();						//economic ranking
		calc.regUtilAid();							//utile aid - finding deficiencies and donating
		calc.worldResetImpExp();					//reset world statistics for import/export
		calc.regClassCoalDmdNProd();				//classification, utils for food import, coal
		calc.regNonrenew();							//nonrenew, import/export
		calc.regOil();								//oil, import/export
		if (firstYr) calc.regSetOilFoundFYR();
		calc.worldPricingEtc();						//pricing
		calc.regDemand();							//demand in indust/agri/advTech/other
		if (!firstYr) calc.regOverheadDemand();		//overhead demand
		calc.regProdDistribution();					//production distribution
		calc.regImpDmd();							//import demand
		if (firstYr) calc.regExpAvailAFYR();		//exports available
		else         calc.regExpAvailA();
		calc.regExpAvailB();
		calc.worldPrepareImpExp();					//set some worldData
		calc.regImports();							//import of food(B), oil, coal, nonrenew and the 5 industrial sectors
		calc.worldSetVarsB();						//reset some vars
		calc.regFoodExpNAid();						//food export and aid
		calc.regOilCoalNonrenewExp();				//export of oil, coal and nonrenew
		calc.regProduction();						//altering production efforts
		calc.regStatUtilsTNI();						//calculating TNI
		calc.regRankByFoodNeed();					//biggest need of food: Nr.0
		calc.regFoodAid();							//distribute and collect food aid
		calc.regDelayedIdleFrac();					//delayed idle industrial fraction
		calc.regOilDist();							//calculate energy (oil) distribution in case of shortage
		calc.regOilType();							//change oil type (only downwards!)
		calc.regStdOfLiving();						//material standard of living
		calc.regPollution();						//industrial decay + coal use + oil use
		calc.regPopulation();						//birth rate / death rate / training
		calc.regStrenClassA();						//limitate strength classification
		if (!firstYr) calc.regStrenClassB();		//limitate strength classification change
		
		//DEBUG 020312
//		Runtime.getRuntime().gc();
	}
	
	
	public void updateUtility(Utility uArg) {
		u = uArg;
		calc.u = uArg;
		sInput.u = uArg;
		sOut.u = uArg;
	}
}
