package logica.sim.io;

import java.util.*;
import java.io.*;
import java.net.*;
import java.util.zip.*;

import logica.utility.*;
import logica.sim.data.*;

public class SimInput {
	private String srcLocation;
	private RegionData[] loadedRegs;

	public Utility u;

	public SimInput(String srcLocation, Utility uArg) {
		u = uArg;

		this.srcLocation = srcLocation;

		try {
			URL srcRegionFile = new URL(srcLocation + "regiondata.zip");
			loadedRegs = getRegions(srcRegionFile);
		}
		catch (MalformedURLException e){
			System.out.println("Bad URL for region data file");
		}
	}


	public void loadRegs(RegionData[] regRefArg)
	{
		int lenMaxT = Math.min(loadedRegs.length, regRefArg.length);

		for (int i = 0; i < lenMaxT; i++) {
			regRefArg[i].name = loadedRegs[i].name;
			regRefArg[i].area = loadedRegs[i].area;
			regRefArg[i].food_surplus = loadedRegs[i].food_surplus;
			regRefArg[i].pop_stat = loadedRegs[i].pop_stat;
			regRefArg[i].pop_inAgri = loadedRegs[i].pop_inAgri;
			regRefArg[i].util_tni = loadedRegs[i].util_tni;
			regRefArg[i].util_avail = loadedRegs[i].util_avail;
			regRefArg[i].nonrenew_fact = loadedRegs[i].nonrenew_fact;
			regRefArg[i].coal_resource = loadedRegs[i].coal_resource;
			regRefArg[i].food_calPerCapDayNeed = loadedRegs[i].food_calPerCapDayNeed;
			regRefArg[i].food_calPerCapDayBY = loadedRegs[i].food_calPerCapDayBY;
			regRefArg[i].pop_nutLvl = loadedRegs[i].pop_nutLvl;
			regRefArg[i].educ_manHourTrainCap = loadedRegs[i].educ_manHourTrainCap;
			regRefArg[i].food_prodGrowthBY = loadedRegs[i].food_prodGrowthBY;
			regRefArg[i].oil_totalReserves = loadedRegs[i].oil_totalReserves;
			regRefArg[i].oil_proven = loadedRegs[i].oil_proven;
			regRefArg[i].oil_type = loadedRegs[i].oil_type - 1; 					//020130: DEBUG!
			regRefArg[i].oil_extrDiffFactor = loadedRegs[i].oil_extrDiffFactor;
			// Util classes - Target
			regRefArg[i].util_targetS_indust = loadedRegs[i].util_targetS_indust;
			regRefArg[i].util_targetS_agri = loadedRegs[i].util_targetS_agri;
			regRefArg[i].util_targetS_advTech = loadedRegs[i].util_targetS_advTech;
			regRefArg[i].util_targetS_other = loadedRegs[i].util_targetS_other;
			// Util classes - Invest Ratio
			regRefArg[i].util_investS_indust = loadedRegs[i].util_investS_indust;
			regRefArg[i].util_investS_agri = loadedRegs[i].util_investS_agri;
			regRefArg[i].util_investS_advTech = loadedRegs[i].util_investS_advTech;
			regRefArg[i].util_investS_other = loadedRegs[i].util_investS_other;
			regRefArg[i].util_investS_overhead = loadedRegs[i].util_investS_overhead;
			regRefArg[i].util_investS_exp = loadedRegs[i].util_investS_exp;
			// Util classes - Possar
			regRefArg[i].imp_possS_indust = loadedRegs[i].imp_possS_indust;
			regRefArg[i].imp_possS_agri = loadedRegs[i].imp_possS_agri;
			regRefArg[i].imp_possS_advTech = loadedRegs[i].imp_possS_advTech;
			regRefArg[i].imp_possS_other = loadedRegs[i].imp_possS_other;

			regRefArg[i].xPos = loadedRegs[i].xPos;
			regRefArg[i].yPos = loadedRegs[i].yPos;
		}
	}


	public RegionData[] getRegions(URL srcUrl)
	{
		try
		{
			System.out.println("reading regiondata from: " + srcUrl.toString());					
			//load from URL
			CSVReader in = new CSVReader(srcUrl.openStream());


			// load as zip
			String urlName = srcUrl.toString();
			if (urlName.endsWith("zip"))
			{
				ZipInputStream zipped_in = new ZipInputStream(srcUrl.openStream());
				ZipEntry entry = zipped_in.getNextEntry();
				System.out.println("reading zipped file:" + entry.getName());
				
				in = new CSVReader(zipped_in);
			}

			// load as Resource
			//CSVReader in = new CSVReader(getResourceStream("/logica/resources/regiondata.csv"));				

			RegionData[] region = new RegionData[ in.countRows() ];
			String[] row = new String[in.countFields()];
			int i = 0;

			while (in.hasNextRow())
			{
				row = in.fetchRow();
				region[i] = new RegionData();
				region[i].name = row[0];
				region[i].area = Float.valueOf(row[1]).floatValue();
				region[i].food_surplus = Float.valueOf(row[2]).floatValue();
				region[i].pop_stat = Float.valueOf(row[3]).floatValue();
				region[i].pop_inAgri = Float.valueOf(row[4]).floatValue();
				region[i].util_tni = Float.valueOf(row[5]).floatValue();
				region[i].util_avail = Float.valueOf(row[6]).floatValue();
				region[i].nonrenew_fact = Float.valueOf(row[7]).floatValue();
				region[i].coal_resource = Float.valueOf(row[8]).floatValue();
				region[i].food_calPerCapDayNeed = Float.valueOf(row[9]).floatValue();
				region[i].food_calPerCapDayBY = Float.valueOf(row[10]).floatValue();
				region[i].pop_nutLvl = Float.valueOf(row[11]).floatValue();
				region[i].educ_manHourTrainCap = Float.valueOf(row[12]).floatValue();
				region[i].food_prodGrowthBY = Float.valueOf(row[13]).floatValue();
				region[i].oil_totalReserves = Float.valueOf(row[14]).floatValue();
				region[i].oil_proven = Float.valueOf(row[15]).floatValue();
				region[i].oil_type = Integer.parseInt(row[16]);
				region[i].oil_extrDiffFactor = Float.valueOf(row[17]).floatValue();

				region[i].util_targetS_indust = Float.valueOf(row[18]).floatValue();
				region[i].util_targetS_agri = Float.valueOf(row[19]).floatValue();
				region[i].util_targetS_advTech = Float.valueOf(row[20]).floatValue();
				region[i].util_targetS_other = Float.valueOf(row[21]).floatValue();

				region[i].util_investS_indust = Float.valueOf(row[22]).floatValue();
				region[i].util_investS_agri = Float.valueOf(row[23]).floatValue();
				region[i].util_investS_advTech = Float.valueOf(row[24]).floatValue();
				region[i].util_investS_other = Float.valueOf(row[25]).floatValue();
				region[i].util_investS_overhead = Float.valueOf(row[26]).floatValue();
				region[i].util_investS_exp = Float.valueOf(row[27]).floatValue();

				region[i].imp_possS_indust = Float.valueOf(row[28]).floatValue();
				region[i].imp_possS_agri = Float.valueOf(row[29]).floatValue();
				region[i].imp_possS_advTech = Float.valueOf(row[30]).floatValue();
				region[i].imp_possS_other = Float.valueOf(row[31]).floatValue();

				region[i].xPos = Float.valueOf(row[32]).floatValue();
				region[i].yPos = Float.valueOf(row[33]).floatValue();

				i++;
			}

			System.out.println("read: "+ i + " countries.");

			return region;
		}
		catch (IOException e)
		{
			System.out.println("Fehler beim ffnen der Datei: " + srcUrl.toString());
			System.out.println(e.toString());			
			return null;
		}
	}



	public int getNumOfRegs() {
		return loadedRegs.length;
	}


	 public InputStream getResourceStream(String resourceName)
	 {
		InputStream istream = null;
		try {
			istream = getClass().getResourceAsStream(resourceName);
		} catch (java.lang.Exception e)
			{ System.out.println(e.toString()); }
		return istream;
	 }

}