package logica.sim.io;

import logica.utility.*;
import logica.sim.data.*;
import logica.sim.*;

import java.util.*;
import java.util.zip.*;
import java.io.*;
import java.net.*;
import java.text.*;


public class UIHandler {

	private uiData[][] ui;
	private uiDataCount[] uiCount;
	private uiData[] uiThisUser;
	private uiData[] mixedUpUI;			//uiThisUser + ui; sepp 020314

	private Year[] data;

	private Utility u;
	private Vars v;
	private URL dataBaseUrl;

	public int numRegs;

	public UIHandler(Year[] dataArg, Vars vArg)
	{
		data = dataArg;
		v = vArg;
		//u = uArg;

		numRegs = v.numRegs;

		ui = new uiData[999][numRegs];
		for (int i = 0; i < 999; i++)
			for (int j = 0; j < numRegs; j++) ui[i][j] = new uiData();

		uiCount = new uiDataCount[numRegs];
		for (int i = 0; i < numRegs; i++) uiCount[i] = new uiDataCount();

		uiThisUser = new uiData[numRegs];
		for (int i = 0; i < numRegs; i++) uiThisUser[i] = new uiData();

		mixedUpUI = new uiData[numRegs];
		for (int i = 0; i < numRegs; i++) mixedUpUI[i] = new uiData();
	}

	public void setPresentYear(int yr)
	{
		v.presentYr = yr + 6;
	}

	public void resetUI()
	{
		for (int i = 0; i < numRegs; i++) uiThisUser[i] = new uiData();
	}



// internal accessor methods

	public float getUserValue(String varName, int regNr) {
		uiData u = uiThisUser[regNr];
		if(varName.equals("utilAid"))                 return u.ui_utilAid;
		else if(varName.equals("foodAid"))            return u.ui_foodAid;
		else if(varName.equals("targetS_industBase")) return u.ui_targetS_industBase;
		else if(varName.equals("targetS_agriEquip"))  return u.ui_targetS_agriEquip;
		else if(varName.equals("targetS_advTech"))    return u.ui_targetS_advTech;
		else if(varName.equals("targetS_other"))      return u.ui_targetS_other;
		else return 0.0F;
	}

	public float getCommonValue(String varName, int regNr, int year) {
		uiData u = ui[year][regNr];
		if(varName.equals("utilAid"))                 return u.ui_utilAid;
		else if(varName.equals("foodAid"))            return u.ui_foodAid;
		else if(varName.equals("targetS_industBase")) return u.ui_targetS_industBase;
		else if(varName.equals("targetS_agriEquip"))  return u.ui_targetS_agriEquip;
		else if(varName.equals("targetS_advTech"))    return u.ui_targetS_advTech;
		else if(varName.equals("targetS_other"))      return u.ui_targetS_other;
		else return 0.0F;
	}

	public float getActualValue(String varName, int regNr, int year) {
		RegionData r = data[year].r[regNr];
		if (varName.equals("utilAid"))                return r.util_avail2Aid;
		else if(varName.equals("foodAid"))            return r.food_surp2Aid;
		else if(varName.equals("targetS_industBase")) return r.util_targetS_indust;
		else if(varName.equals("targetS_agriEquip"))  return r.util_targetS_agri;
		else if(varName.equals("targetS_advTech"))    return r.util_targetS_advTech;
		else if(varName.equals("targetS_other"))      return r.util_targetS_other;
		else return 0.0F;
	}

	public int getCount(String varName, int regNr) {
		uiDataCount u = uiCount[regNr];
		if (varName.equals("utilAid"))                return u.ui_utilAid;
		else if(varName.equals("foodAid"))            return u.ui_foodAid;
		else if(varName.equals("targetS_industBase")) return u.ui_targetS_industBase;
		else if(varName.equals("targetS_agriEquip"))  return u.ui_targetS_agriEquip;
		else if(varName.equals("targetS_advTech"))    return u.ui_targetS_advTech;
		else if(varName.equals("targetS_other"))      return u.ui_targetS_other;
		else return 0;
	}


	//external accessor methods
	public float getSliderVal(String varName, int regNr) {
		return getActualValue(varName, regNr, v.presentYr);
	}

	public float getUserSliderVal(String varName, int regNr) {
		float retValue = 0.0F;
		if (getUserValue(varName, regNr) == 0.0F) {
			return getActualValue(varName, regNr, v.presentYr - 1);
		}
		else {
			retValue = getActualValue(varName, regNr, v.presentYr - 1);
			retValue += calcUIVar(getLimits(varName, regNr, v.presentYr - 1),
			                      getActualValue(varName, regNr, v.presentYr - 1),
								  getUserValue(varName, regNr));
			return retValue;
		}
	}



	public float[] getLimits(String varName, int regNrArg, int yrArg) {
		float[] limits = { 0.0F, 1.0F };
		return limits;
	}


	public float[][] getDistLimits(String varName, int regNrArg) {
		float[][] limits =  {{0.0F, 1.0F},
		        		      {0.0F, 1.0F},
				              {0.0F, 1.0F},
				              {0.0F, 1.0F}};
		return limits;
	}


	public void submitValue(String varName, int regNr, float value)
	{
		// sepp 020314
		float uiValue = absolute2UIValue(varName, regNr, value);
		if (varName.equals("utilAid")) uiThisUser[regNr].ui_utilAid = uiValue;
		else if (varName.equals("foodAid")) uiThisUser[regNr].ui_foodAid = uiValue;
	}

	// sepp 020317
	public void submitDistValue(String varName, int regNr, float values[])
	{
		float[] uiValues = new float[4];

		if (varName.equals("targetS"))
		{
			if ((values[0] == 0.0F) &
			    (values[1] == 0.0F) &
				(values[2] == 0.0F) &
				(values[3] == 0.0F)) {
					
				values[0] = 0.25F;
				values[1] = 0.25F;
				values[2] = 0.25F;
				values[3] = 0.25F;
			}
			
			uiValues[0] = absolute2UIValue("targetS_industBase", regNr, values[0]);
			uiValues[1] = absolute2UIValue("targetS_agriEquip", regNr, values[1]);
			uiValues[2] = absolute2UIValue("targetS_advTech", regNr, values[2]);
			uiValues[3] = absolute2UIValue("targetS_other", regNr, values[3]);

			uiThisUser[regNr].ui_targetS_industBase = uiValues[0];
			uiThisUser[regNr].ui_targetS_agriEquip = uiValues[1];
			uiThisUser[regNr].ui_targetS_advTech = uiValues[2];
			uiThisUser[regNr].ui_targetS_other = uiValues[3];
		}
	}

	//utility-method for submitValue
	// sepp 020314
	public float absolute2UIValue (String varName, int regNr, float userValue) {
		float refValue = getActualValue(varName, regNr, v.presentYr - 1);
		float[] limits = getLimits(varName, regNr, v.presentYr - 1);
		float uiValue = deNaN((userValue - refValue) / ((userValue > refValue) ?
		                                               (limits[1] - refValue) :
		                                               (refValue - limits[0])));
		return uiValue;
	}


	//incorporates UI in vars
	// sepp 020313
	public void incorporateUI() {

		if (v.calcYr == v.presentYr) {
			mixUpUI();
		}
		else {
			mixedUpUI = ui[v.calcYr];
		}

		int year = v.calcYr;

		RegionData r;
		float sum;

		for (int i = 0; i < v.numRegs; i++) {
			r = data[v.calcYr].r[i];
			r.util_avail2Aid += calcUIVar(getLimits("utilAid", i, year),
			                              r.util_avail2Aid,
			                              mixedUpUI[i].ui_utilAid);

			r.food_surp2Aid += calcUIVar(getLimits("foodAid", i, year),
			                             r.food_surp2Aid,
			                             mixedUpUI[i].ui_foodAid);

			//util_targetS

			r.util_targetS_indust +=  calcUIVar(getLimits("targetS_industBase", i, year),
			                             r.util_targetS_indust,
			                             mixedUpUI[i].ui_targetS_industBase);

			r.util_targetS_agri +=    calcUIVar(getLimits("targetS_agriEquip", i, year),
			                             r.util_targetS_agri,
			                             mixedUpUI[i].ui_targetS_agriEquip);

			r.util_targetS_advTech += calcUIVar(getLimits("targetS_advTech", i, year),
			                             r.util_targetS_advTech,
			                             mixedUpUI[i].ui_targetS_advTech);

			r.util_targetS_other +=   calcUIVar(getLimits("targetS_other", i, year),
			                             r.util_targetS_other,
			                             mixedUpUI[i].ui_targetS_other);

			//normalize...
			sum = r.util_targetS_indust + r.util_targetS_agri +
			      r.util_targetS_advTech + r.util_targetS_other;
			
			sum = Math.max(sum, 0.01F);

			r.util_targetS_indust /= sum;
			r.util_targetS_agri /= sum;
			r.util_targetS_advTech /= sum;
			r.util_targetS_other /= sum;
		}
	}

	//utility-method for incorporateUI
	// calculates the amount of difference from old controller-variable value
	// sepp 020313
	public float calcUIVar(float[] limits, float value, float uiValue) {
		if (uiValue > 0.0F) return (limits[1] - value) * uiValue;
		else return (value - limits[0]) * uiValue;
	}


	//mixes up userUI with common UI
	// sepp 020314
	public void mixUpUI() {
		for (int i = 0; i < v.numRegs; i++) {
			mixedUpUI[i].ui_utilAid = mixValue("utilAid", i);
			mixedUpUI[i].ui_foodAid = mixValue("foodAid", i);
			// mash 020402 korr.
			mixedUpUI[i].ui_targetS_industBase = mixValue("targetS_industBase", i);
			mixedUpUI[i].ui_targetS_agriEquip = mixValue("targetS_agriEquip", i);
			mixedUpUI[i].ui_targetS_advTech = mixValue("targetS_advTech", i);
			mixedUpUI[i].ui_targetS_other = mixValue("targetS_other", i);
		}
	}

	//utiliy-method for mixUpUI
	// sepp 020314
	public float mixValue(String varName, int regNr) {
		int uiC = getCount(varName, regNr);
		float mixedValue;
		if ((getUserValue(varName, regNr) != 0.0F) &
		    (getCount(varName, regNr) != 0)) {
			mixedValue =  (getCommonValue(varName, regNr, v.calcYr) * uiC
			               + getUserValue(varName, regNr)) / (uiC + 1);
		}
		else if (getCount(varName, regNr) == 0) {
			mixedValue = getUserValue(varName, regNr);
		}
		else {
			mixedValue = getCommonValue(varName, regNr, v.calcYr);
		}
		return mixedValue;
	}


	public float deNaN(float value) {
		if (Float.isNaN(value)) return 0.0F;
		else return value;
	}
	

	public void getUIHistory(URL srcUrl)
	{
		System.out.println("reading history from: " + srcUrl.toString());
		try
		{
			// load from URL
			CSVReader in = new CSVReader(new GZIPInputStream(srcUrl.openStream()));

			String[] row = new String[in.countFields()];

			while (in.hasNextRow())
			{
				row = in.fetchRow();
				int year = Integer.parseInt(row[0]) + 6;
				int regionNr = Integer.parseInt(row[1]);

				ui[year][regionNr].ui_utilAid = Float.valueOf(row[2]).floatValue();
				ui[year][regionNr].ui_foodAid = Float.valueOf(row[3]).floatValue();
				ui[year][regionNr].ui_targetS_industBase = Float.valueOf(row[4]).floatValue();
				ui[year][regionNr].ui_targetS_agriEquip = Float.valueOf(row[5]).floatValue();
				ui[year][regionNr].ui_targetS_advTech = Float.valueOf(row[6]).floatValue();
				ui[year][regionNr].ui_targetS_other = Float.valueOf(row[7]).floatValue();
				//ui[year][regionNr].ui_immigrMax = Float.valueOf(row[8]).floatValue();
			}

			System.out.println("read UIhistory.");

		}
		catch (IOException e)
		{
			System.out.println("Fehler beim ffnen der Datei: " + srcUrl.toString());
			System.out.println(e.toString());
		}
	}



	public void getUICount(URL srcUrl)
	{
		System.out.println("reading historyCount from: " + srcUrl.toString());
		try
		{
			// load from URL
			CSVReader in = new CSVReader(srcUrl.openStream());

			String[] row = new String[in.countFields()];

			while (in.hasNextRow())
			{
				row = in.fetchRow();
				int regionNr = Integer.parseInt(row[0]);
				uiCount[regionNr].ui_utilAid = Integer.parseInt(row[1]);
				uiCount[regionNr].ui_foodAid = Integer.parseInt(row[2]);
				uiCount[regionNr].ui_targetS_industBase = Integer.parseInt(row[3]);
				uiCount[regionNr].ui_targetS_agriEquip = Integer.parseInt(row[3]);
				uiCount[regionNr].ui_targetS_advTech = Integer.parseInt(row[3]);
				uiCount[regionNr].ui_targetS_other = Integer.parseInt(row[3]);
				//ui[year][regionNr].ui_immigrMax = Integer.parseInt(row[6]);
			}

		System.out.println("read actual UICount");

		}
		catch (IOException e)
		{
		System.out.println("Fehler beim ffnen der Datei: " + srcUrl.toString());
		}
	}




	public String sendToServer(URL url, String sessionID, int yr)
	{
		try
		{
			System.out.println("sending UIdata to: " + url.toString());


	    		URLConnection urlConn;
    			DataOutputStream printout;
    			DataInputStream input;

			urlConn = url.openConnection();
			urlConn.setDoInput (true);
			urlConn.setDoOutput (true);
			urlConn.setUseCaches (false);
			urlConn.setRequestProperty("Content-Type", "application/x-www-form-urlencoded");

			// Send POST output.
			printout = new DataOutputStream (urlConn.getOutputStream ());


			String postString = "sid=" + URLEncoder.encode(sessionID) + "&actYear="+yr;
			//String postString = "sid=" + sessionID + "&actYear="+yr;

			String data_String = "";
			DecimalFormat df = new DecimalFormat("#0.00");			

			for(int i=0; i < uiThisUser.length; i++)
			{
				if (uiThisUser[i].isSet())
				{
					data_String += i + ";";
					data_String += df.format(uiThisUser[i].ui_utilAid) + ";";
					data_String += df.format(uiThisUser[i].ui_foodAid) + ";";
					data_String += df.format(uiThisUser[i].ui_targetS_industBase) + ";";
					data_String += df.format(uiThisUser[i].ui_targetS_agriEquip) + ";";
					data_String += df.format(uiThisUser[i].ui_targetS_advTech) + ";";
					data_String += df.format(uiThisUser[i].ui_targetS_other) + "\n";
				}
			}

			if (data_String != "")
			{
				postString += "&ui_data=" + URLEncoder.encode(data_String);

/*DEBUG*/			System.out.println("postString: " + postString);
/*DEBUG*/			//System.out.println(URLDecoder.decode(data_String));

				printout.writeBytes(postString);
				printout.flush();
				printout.close();

				// Get response data.
				input = new DataInputStream (urlConn.getInputStream ());

				Properties returnProps = new Properties();
				returnProps.load(input);

				boolean success = (new Boolean( returnProps.getProperty("success","false") ) ).booleanValue();
				boolean valid = (new Boolean( returnProps.getProperty("valid","true") ) ).booleanValue();
				String error = returnProps.getProperty("error","");
				input.close ();

/*DEBUG*/			System.out.println("success: " + success);
/*DEBUG*/			System.out.println("error msg: " + error);
				return error;
			}
			else
			{
				return "you didn't change anything. nothing sended.";
			}

		}
		catch (Exception e)
		{
			System.out.println("Fehler beim Senden an: " + url.toString() + " : " + e.toString());
			return "unkown error submitting";
		}

	}
		
}
